/*
 *******************************************************************************
 *
 *  Copyright 2023 RIEGL Laser Measurement Systems
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 *  SPDX-License-Identifier: Apache-2.0
 *
 *******************************************************************************
 */
/*!
 *******************************************************************************
 *
 * \author  RIEGL LMS GmbH, Austria
 * \brief   Description of RIEGL RDB 2 database meta data items.
 * \version 2015-10-27/AW: Initial version
 * \version 2015-11-25/AW: Item "Geo Tag" added
 * \version 2016-10-27/AW: Item "Voxel Information" added
 * \version 2016-11-17/AW: Item "Voxel Information" updated
 * \version 2016-12-12/AW: Item "Range Statistics" added
 * \version 2017-03-08/AW: Item "Plane Patch Statistics" added
 * \version 2017-04-05/AW: Items "Atmosphere" and "Geometric Scale Factor" added
 * \version 2017-08-22/AW: Items for waveform sample block and value files added
 * \version 2017-10-24/AW: Item "Gaussian Decomposition" added
 * \version 2017-11-13/AW: Item "riegl.scan_pattern" updated
 * \version 2017-11-21/AW: Item "riegl.trajectory_info" added
 * \version 2018-01-11/AW: Item "riegl.beam_geometry" added
 * \version 2018-01-15/AW: Item "riegl.reflectance_calculation" added
 * \version 2018-01-15/AW: Item "riegl.near_range_correction" added
 * \version 2018-01-15/AW: Item "riegl.device_geometry" added
 * \version 2018-02-13/AW: Item "riegl.notch_filter" added
 * \version 2018-03-08/AW: Item "riegl.window_echo_correction" added
 * \version 2018-03-15/AW: Item "riegl.pulse_position_modulation" added
 * \version 2018-05-24/AW: Item "riegl.pixel_info" added
 * \version 2018-06-08/AW: Item "riegl.shot_info" added
 * \version 2018-06-08/AW: Item "riegl.echo_info" added
 * \version 2018-06-14/AW: Item "riegl.mta_settings" added
 * \version 2018-06-14/AW: Item "riegl.receiver_internals" added
 * \version 2018-06-14/AW: Item "riegl.device_output_limits" added
 * \version 2018-06-26/AW: Schema: replace "number" with "integer" if applicable
 * \version 2018-07-09/AW: Item "riegl.pose_estimation" added
 * \version 2018-07-09/AW: Item "riegl.pose_sensors" added
 * \version 2018-09-20/AW: Item "riegl.pointcloud_info" added
 * \version 2018-11-08/AW: Item "riegl.scan_pattern" updated
 * \version 2018-11-13/AW: Item "riegl.receiver_internals" updated
 * \version 2019-03-06/AW: Item "riegl.receiver_internals" updated
 * \version 2019-03-21/AW: Item "riegl.device_geometry" updated
 * \version 2019-04-15/AW: Item "riegl.point_attribute_groups" added
 * \version 2019-04-30/AW: Item "riegl.waveform_settings" added
 * \version 2019-10-03/AW: Item "riegl.angular_notch_filter" added
 * \version 2019-10-03/AW: Item "riegl.noise_estimates" added
 * \version 2019-10-25/AW: Item "riegl.window_analysis" added
 * \version 2019-11-06/AW: Item "riegl.georeferencing_parameters" added
 * \version 2019-11-27/AW: Item "riegl.plane_patch_matching" added
 * \version 2019-12-13/AW: Items for tie-/control objects added
 * \version 2019-12-19/AW: Items for tie-/control objects added
 * \version 2020-02-04/AW: Item "riegl.detection_probability" added
 * \version 2020-02-04/AW: Item "riegl.licenses" added
 * \version 2020-04-27/AW: Item "riegl.waveform_info" updated (schema+example)
 * \version 2020-09-03/AW: Item "riegl.reflectance_correction" added
 * \version 2020-09-10/AW: Item "riegl.device_geometry_passive_channel" added
 * \version 2020-09-25/AW: Item "riegl.georeferencing_parameters" updated
 * \version 2020-09-25/AW: Item "riegl.trajectory_info" updated
 * \version 2020-09-29/AW: Item "riegl.temperature_calculation" added
 * \version 2020-10-23/AW: Item "riegl.exponential_decomposition" added (#3709)
 * \version 2020-11-30/AW: Item "riegl.notch_filter" updated (schema)
 * \version 2020-12-02/AW: Item "riegl.georeferencing_parameters" updated (schema)
 * \version 2021-02-02/AW: Item "riegl.plane_slope_class_info" added (rdbplanes/#7)
 * \version 2021-02-16/AW: Item "riegl.device_output_limits" updated (schema, #3811)
 * \version 2021-03-03/AW: Item "riegl.exponential_decomposition" updated (schema, #3822)
 * \version 2021-03-03/AW: Item "riegl.waveform_averaging_settings" added (#3821)
 * \version 2021-04-01/AW: Item "riegl.voxel_info" updated (schema, #3854)
 * \version 2021-04-16/AW: Item "riegl.voxel_info" updated (schema, #3866)
 * \version 2021-09-30/AW: Item "riegl.waveform_info" updated (schema+example, #4016)
 * \version 2021-10-04/AW: Improved spelling of the descriptions of some items
 * \version 2021-11-04/AW: Rename "riegl.record_names" to "riegl.item_names" (#4034)
 * \version 2022-03-11/AW: Item "riegl.devices" added (#4162)
 * \version 2022-03-14/AW: Item "riegl.stored_filters" added (#4164)
 * \version 2022-09-20/AW: Fix typos in schema of "riegl.georeferencing_parameters"
 * \version 2022-09-30/AW: Item "riegl.system_description" added (#4368)
 * \version 2022-10-10/AW: Correction of typos in the titles of some entries
 * \version 2022-10-20/AW: Item "riegl.stored_filters" updated (example, #4164)
 * \version 2023-01-19/AW: Item "riegl.ttip_configuration" added (#4449)
 *
 *******************************************************************************
 */

#ifndef RDB_354CFFB3_BDAA_4E38_98C8_9C09D0C0AD87
#define RDB_354CFFB3_BDAA_4E38_98C8_9C09D0C0AD87

#include <vector>
#include <string>
#include <cstdlib>
#include <cstdint>

namespace riegl {
namespace rdb {
namespace pointcloud {

// Meta data item definition version
const std::string RDB_META_DATA_ITEMS_VERSION = "1.1.54";
const std::string RDB_META_DATA_ITEMS_DATE = "2023-01-19";

// Meta data item states
const unsigned char RDB_STATUS_REQUIRED = 1; // mandatory item
const unsigned char RDB_STATUS_OPTIONAL = 2; // optional item

// Container record for meta data item details
struct MetaDataItemInfo
{
    std::string   name;
    std::string   title;
    std::string   description;
    unsigned char status;
    std::string   schema;
    std::string   example;

    operator const std::string&() const { return name; }
};

// Angular notch filter parameters for window glass echoes
const MetaDataItemInfo RDB_RIEGL_ANGULAR_NOTCH_FILTER = {
    /* name        */ "riegl.angular_notch_filter",
    /* title       */ "Angular Notch Filter",
    /* description */ "Angular notch filter parameters for window glass echoes",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Angular Notch Filter\",\"type\":\"object\",\"$schema\":\"http://json"
    "-schema.org/draft-04/schema#\",\"properties\":{\"range_mean\":{\"items\":{\"ty"
    "pe\":\"number\"},\"description\":\"Mean range [m]\",\"type\":\"array\"},\"amplitud"
    "e_mean\":{\"items\":{\"type\":\"number\"},\"description\":\"Mean amplitude [dB]\""
    ",\"type\":\"array\"},\"angle\":{\"items\":{\"type\":\"number\"},\"description\":\"Ang"
    "le [deg]\",\"type\":\"array\"}},\"description\":\"Angular notch filter "
    "parameters for window glass "
    "echoes\",\"required\":[\"angle\",\"range_mean\",\"amplitude_mean\"]}"
    ,
    /* example: */
    "{\"range_mean\":[0.094,0.094,0.09075,0.08675,0.0815,0.0775,0.074,0.071,0"
    ".068,0.0675,0.06475],\"amplitude_mean\":[3.780913,3.780913,3.480913,3.12"
    "0913,2.850913,2.720913,2.680913,2.610913,2.530913,2.570913,2.570913],\""
    "angle\":[14.0,15.0,16.0,17.0,18.0,19.0,20.0,21.0,22.0,23.0,24.0]}"
};

// Atmospheric parameters
const MetaDataItemInfo RDB_RIEGL_ATMOSPHERE = {
    /* name        */ "riegl.atmosphere",
    /* title       */ "Atmosphere",
    /* description */ "Atmospheric parameters",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Atmospheric Parameters\",\"type\":\"object\",\"$schema\":\"http://js"
    "on-schema.org/draft-04/schema#\",\"properties\":{\"pressure\":{\"description"
    "\":\"Pressure along measurment path "
    "[mbar]\",\"type\":\"number\"},\"wavelength\":{\"description\":\"Laser wavelength"
    " [nm]\",\"type\":\"number\"},\"attenuation\":{\"description\":\"Atmospheric "
    "attenuation [1/km]\",\"type\":\"number\"},\"amsl\":{\"description\":\"Height "
    "above mean sea level (AMSL) "
    "[m]\",\"type\":\"number\"},\"pressure_sl\":{\"description\":\"Atmospheric "
    "pressure at sea level "
    "[mbar]\",\"type\":\"number\"},\"group_velocity\":{\"description\":\"Group "
    "velocity of laser beam "
    "[m/s]\",\"type\":\"number\"},\"rel_humidity\":{\"description\":\"Relative "
    "humidity along measurement path "
    "[%]\",\"type\":\"number\"},\"temperature\":{\"description\":\"Temperature along "
    "measurement path "
    "[\\u00b0C]\",\"type\":\"number\"}},\"description\":\"Atmospheric parameters\",\"r"
    "equired\":[\"temperature\",\"pressure\",\"rel_humidity\",\"pressure_sl\",\"amsl\""
    ",\"group_velocity\",\"attenuation\",\"wavelength\"]}"
    ,
    /* example: */
    "{\"pressure\":970,\"wavelength\":1550,\"attenuation\":0.028125,\"amsl\":0,\"pre"
    "ssure_sl\":970,\"group_velocity\":299711000.0,\"rel_humidity\":63,\"temperat"
    "ure\":7}"
};

// Laser beam geometry details
const MetaDataItemInfo RDB_RIEGL_BEAM_GEOMETRY = {
    /* name        */ "riegl.beam_geometry",
    /* title       */ "Beam Geometry",
    /* description */ "Laser beam geometry details",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Beam Geometry\",\"type\":\"object\",\"$schema\":\"http://json-schema"
    ".org/draft-04/schema#\",\"properties\":{\"beam_divergence\":{\"exclusiveMini"
    "mum\":false,\"type\":\"number\",\"description\":\"Beam divergence in far field"
    " [rad]\",\"minimum\":0},\"beam_exit_diameter\":{\"exclusiveMinimum\":false,\"t"
    "ype\":\"number\",\"description\":\"Beam width at exit aperture "
    "[m]\",\"minimum\":0}},\"description\":\"Laser beam geometry "
    "details\",\"required\":[\"beam_exit_diameter\",\"beam_divergence\"]}"
    ,
    /* example: */
    "{\"beam_divergence\":0.0003,\"beam_exit_diameter\":0.0072}"
};

// List of control object type definitions
const MetaDataItemInfo RDB_RIEGL_CONTROL_OBJECT_CATALOG = {
    /* name        */ "riegl.control_object_catalog",
    /* title       */ "Control Object Catalog",
    /* description */ "List of control object type definitions",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Control Object Catalog\",\"type\":\"object\",\"$schema\":\"http://js"
    "on-schema.org/draft-04/schema#\",\"properties\":{\"types\":{\"items\":{\"oneOf"
    "\":[{\"$ref\":\"#/definitions/rectangle\"},{\"$ref\":\"#/definitions/checkerbo"
    "ard2x2\"},{\"$ref\":\"#/definitions/chevron\"},{\"$ref\":\"#/definitions/circu"
    "lar_disk\"},{\"$ref\":\"#/definitions/cylinder\"},{\"$ref\":\"#/definitions/sp"
    "here\"},{\"$ref\":\"#/definitions/round_corner_cube_prism\"}],\"type\":\"objec"
    "t\"},\"uniqueItems\":true,\"type\":\"array\"}},\"required\":[\"types\"],\"descript"
    "ion\":\"List of control object type definitions\",\"definitions\":{\"round_c"
    "orner_cube_prism\":{\"type\":\"object\",\"description\":\"round corner cube pr"
    "ism\",\"allOf\":[{\"$ref\":\"#/definitions/common\"},{\"properties\":{\"offset\":"
    "{\"type\":\"number\",\"description\":\"offset in meters, e.g. reflector "
    "constant (optional)\"},\"shape\":{\"type\":\"string\",\"description\":\"shape id"
    "entifier\",\"enum\":[\"round_corner_cube_prism\"]},\"diameter\":{\"exclusiveMi"
    "nimum\":true,\"type\":\"number\",\"description\":\"diameter in "
    "meters\",\"minimum\":0}},\"type\":\"object\",\"description\":\"round corner cube"
    " prism specific properties\",\"required\":[\"shape\",\"diameter\"]}]},\"cylind"
    "er\":{\"type\":\"object\",\"description\":\"cylinder\",\"allOf\":[{\"$ref\":\"#/defi"
    "nitions/common\"},{\"properties\":{\"height\":{\"exclusiveMinimum\":true,\"typ"
    "e\":\"number\",\"description\":\"height in "
    "meters\",\"minimum\":0},\"shape\":{\"type\":\"string\",\"description\":\"shape ide"
    "ntifier\",\"enum\":[\"cylinder\"]},\"diameter\":{\"exclusiveMinimum\":true,\"typ"
    "e\":\"number\",\"description\":\"diameter in "
    "meters\",\"minimum\":0}},\"type\":\"object\",\"description\":\"cylinder specific"
    " properties\",\"required\":[\"shape\",\"diameter\",\"height\"]}]},\"rectangle\":{"
    "\"type\":\"object\",\"description\":\"rectangle\",\"allOf\":[{\"$ref\":\"#/definiti"
    "ons/common\"},{\"properties\":{\"width\":{\"exclusiveMinimum\":true,\"type\":\"n"
    "umber\",\"description\":\"width in meters\",\"minimum\":0},\"shape\":{\"enum\":[\""
    "rectangle\"],\"type\":\"string\"},\"length\":{\"exclusiveMinimum\":true,\"type\":"
    "\"number\",\"description\":\"length in "
    "meters\",\"minimum\":0}},\"type\":\"object\",\"description\":\"rectangle "
    "specific properties\",\"required\":[\"shape\",\"length\",\"width\"]}]},\"chevron"
    "\":{\"type\":\"object\",\"description\":\"chevron\",\"allOf\":[{\"$ref\":\"#/definit"
    "ions/common\"},{\"properties\":{\"thickness\":{\"exclusiveMinimum\":true,\"typ"
    "e\":\"number\",\"description\":\"thickness in "
    "meters\",\"minimum\":0},\"shape\":{\"type\":\"string\",\"description\":\"shape ide"
    "ntifier\",\"enum\":[\"chevron\"]},\"outside_edge_length\":{\"exclusiveMinimum\""
    ":true,\"type\":\"number\",\"description\":\"length of the two outer edges in "
    "meters\",\"minimum\":0}},\"type\":\"object\",\"description\":\"chevron specific "
    "properties\",\"required\":[\"shape\",\"outside_edge_length\",\"thickness\"]}]},"
    "\"circular_disk\":{\"type\":\"object\",\"description\":\"circular disk\",\"allOf\""
    ":[{\"$ref\":\"#/definitions/common\"},{\"properties\":{\"offset\":{\"type\":\"num"
    "ber\",\"description\":\"offset in meters, e.g. reflector constant "
    "(optional)\"},\"shape\":{\"type\":\"string\",\"description\":\"shape identifier\""
    ",\"enum\":[\"circular_disk\"]},\"diameter\":{\"exclusiveMinimum\":true,\"type\":"
    "\"number\",\"description\":\"diameter in "
    "meters\",\"minimum\":0}},\"type\":\"object\",\"description\":\"circular disk "
    "specific properties\",\"required\":[\"shape\",\"diameter\"]}]},\"common\":{\"pro"
    "perties\":{\"name\":{\"minLength\":3,\"type\":\"string\",\"description\":\"unique "
    "type "
    "identifier\"},\"surface_type\":{\"type\":\"string\",\"description\":\"surface "
    "material type\",\"enum\":[\"retro_reflective_foil\",\"diffuse\"]},\"shape\":{\"t"
    "ype\":\"string\",\"description\":\"shape identifier\",\"enum\":[\"rectangle\",\"ch"
    "eckerboard2x2\",\"chevron\",\"circular_disk\",\"cylinder\",\"sphere\",\"round_co"
    "rner_cube_prism\"]},\"description\":{\"type\":\"string\",\"description\":\"strin"
    "g describing the object\"}},\"type\":\"object\",\"description\":\"common "
    "object properties\",\"required\":[\"name\",\"shape\"]},\"checkerboard2x2\":{\"ty"
    "pe\":\"object\",\"description\":\"checkerboard 2 by 2\",\"allOf\":[{\"$ref\":\"#/d"
    "efinitions/common\"},{\"properties\":{\"square_length\":{\"exclusiveMinimum\""
    ":true,\"type\":\"number\",\"description\":\"length of a square of the "
    "checkerboard in "
    "meters\",\"minimum\":0},\"shape\":{\"type\":\"string\",\"description\":\"shape ide"
    "ntifier\",\"enum\":[\"checkerboard2x2\"]}},\"type\":\"object\",\"description\":\"c"
    "heckerboard specific properties\",\"required\":[\"shape\",\"square_length\"]}"
    "]},\"sphere\":{\"type\":\"object\",\"description\":\"sphere\",\"allOf\":[{\"$ref\":\""
    "#/definitions/common\"},{\"properties\":{\"shape\":{\"type\":\"string\",\"descri"
    "ption\":\"shape identifier\",\"enum\":[\"sphere\"]},\"diameter\":{\"exclusiveMin"
    "imum\":true,\"type\":\"number\",\"description\":\"diameter in "
    "meters\",\"minimum\":0}},\"type\":\"object\",\"description\":\"sphere specific "
    "properties\",\"required\":[\"shape\",\"diameter\"]}]}}}"
    ,
    /* example: */
    "{\"comments\":[\"This file contains a list of control object types (aka. "
    "'catalog').\",\"Each type is described by an object,\",\"which must "
    "contain at least the following parameters:\",\"  - name: unique "
    "identifier of the type\",\"  - shape: one of the following supported "
    "shapes:\",\"      - rectangle\",\"      - checkerboard2x2\",\"      - "
    "chevron\",\"      - circular_disk\",\"      - cylinder\",\"      - sphere\",\""
    "      - round_corner_cube_prism\",\"Depending on 'shape', the following "
    "parameters must/may be specified:\",\"  - rectangle:\",\"      - length: "
    "length in meters\",\"      - width: width in meters\",\"  - "
    "checkerboard2x2:\",\"      - square_length: length of a square of the "
    "checkerboard in meters\",\"  - circular_disk:\",\"      - diameter: "
    "diameter in meters\",\"      - offset: offset in meters, e.g. reflector "
    "constant (optional)\",\"  - chevron:\",\"      - outside_edge_length: "
    "length of the two outer edges in meters\",\"      - thickness: thickness"
    " in meters\",\"  - cylinder:\",\"      - diameter: diameter in meters\",\""
    "      - height:  height in meters\",\"  - sphere:\",\"      - diameter: "
    "diameter in meters\",\"  - round_corner_cube_prism:\",\"      - diameter: "
    "diameter in meters\",\"      - offset: offset in meters, e.g. reflector "
    "constant (optional)\",\"Optional parameters:\",\"    - description: string"
    " describing the object\",\"    - surface_type: surface material type "
    "(either 'retro_reflective_foil' or "
    "'diffuse')\"],\"types\":[{\"name\":\"Rectangle "
    "60x30\",\"width\":0.3,\"shape\":\"rectangle\",\"description\":\"Rectangle (60cm "
    "x 30cm)\",\"length\":0.6},{\"name\":\"Rectangle "
    "80x40\",\"width\":0.4,\"shape\":\"rectangle\",\"description\":\"Rectangle (80cm "
    "x 40cm)\",\"length\":0.8},{\"name\":\"Checkerboard2x2 30\",\"square_length\":0."
    "3,\"shape\":\"checkerboard2x2\",\"description\":\"Checkerboard (square "
    "length: 30cm)\"},{\"name\":\"Checkerboard2x2 50\",\"square_length\":0.5,\"shap"
    "e\":\"checkerboard2x2\",\"description\":\"Checkerboard (square length: "
    "50cm)\"},{\"name\":\"Chevron 24''/4''\",\"thickness\":0.1016,\"outside_edge_le"
    "ngth\":0.6096,\"shape\":\"chevron\",\"description\":\"Chevron (a=24''; "
    "b=4'')\"},{\"name\":\"Circular Disk 50\",\"surface_type\":\"diffuse\",\"diameter"
    "\":0.5,\"shape\":\"circular_disk\",\"description\":\" Circular Disk (diameter:"
    " 50cm)\"},{\"name\":\"RIEGL flat reflector 50 mm\",\"surface_type\":\"retro_re"
    "flective_foil\",\"shape\":\"circular_disk\",\"diameter\":0.05,\"offset\":0.0,\"d"
    "escription\":\"flat circular reflector from retro-reflective "
    "foil\"},{\"name\":\"RIEGL flat reflector 100 mm\",\"surface_type\":\"retro_ref"
    "lective_foil\",\"shape\":\"circular_disk\",\"diameter\":0.1,\"offset\":0.0,\"des"
    "cription\":\"flat circular reflector from retro-reflective "
    "foil\"},{\"name\":\"RIEGL flat reflector 150 mm\",\"surface_type\":\"retro_ref"
    "lective_foil\",\"shape\":\"circular_disk\",\"diameter\":0.15,\"offset\":0.0,\"de"
    "scription\":\"flat circular reflector from retro-reflective "
    "foil\"},{\"name\":\"RIEGL cylindrical reflector 50 "
    "mm\",\"surface_type\":\"retro_reflective_foil\",\"description\":\"cylindrical "
    "reflector from retro-reflective "
    "foil\",\"shape\":\"cylinder\",\"diameter\":0.05,\"height\":0.05},{\"name\":\"RIEGL"
    " cylindrical reflector 100 "
    "mm\",\"surface_type\":\"retro_reflective_foil\",\"description\":\"cylindrical "
    "reflector from retro-reflective "
    "foil\",\"shape\":\"cylinder\",\"diameter\":0.1,\"height\":0.1},{\"name\":\"Sphere "
    "200 mm\",\"diameter\":0.2,\"shape\":\"sphere\",\"description\":\"Sphere "
    "(diameter: 200 mm)\"},{\"name\":\"Corner Cube Prism 50 mm\",\"offset\":0.0,\"d"
    "iameter\":0.05,\"shape\":\"round_corner_cube_prism\",\"description\":\"round "
    "corner cube prism\"}]}"
};

// Details about the control object reference file
const MetaDataItemInfo RDB_RIEGL_CONTROL_OBJECT_REFERENCE_FILE = {
    /* name        */ "riegl.control_object_reference_file",
    /* title       */ "Control Object Reference file",
    /* description */ "Details about the control object reference file",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"properties\":{\"re"
    "ference_file\":{\"properties\":{\"file_uuid\":{\"type\":\"string\",\"description"
    "\":\"Control object file's Universally Unique Identifier (RFC "
    "4122)\"},\"file_path\":{\"type\":\"string\",\"description\":\"Path of the "
    "control object file relative to referring "
    "file\"}},\"description\":\"Reference to a control object file\",\"type\":\"obj"
    "ect\",\"required\":[\"file_uuid\",\"file_path\"]}},\"title\":\"Control Object "
    "Reference file\",\"description\":\"Details about the control object "
    "reference file\",\"type\":\"object\"}"
    ,
    /* example: */
    "{\"reference_file\":{\"file_uuid\":\"810f5d57-eccf-49ed-b07a-0cdd109b4213\","
    "\"file_path\":\"../../../10_CONTROL_OBJECTS/ControlPoints.cpx\"}}"
};

// Detection probability as a function of amplitude
const MetaDataItemInfo RDB_RIEGL_DETECTION_PROBABILITY = {
    /* name        */ "riegl.detection_probability",
    /* title       */ "Detection Probability",
    /* description */ "Detection probability as a function of amplitude",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Detection Probability\",\"type\":\"object\",\"$schema\":\"http://jso"
    "n-schema.org/draft-04/schema#\",\"properties\":{\"detection_probability\":{"
    "\"items\":{\"type\":\"number\"},\"description\":\"Detection probability [0..1]\""
    ",\"type\":\"array\"},\"amplitude\":{\"items\":{\"type\":\"number\"},\"description\":"
    "\"Amplitude [dB]\",\"type\":\"array\"}},\"description\":\"Detection probability"
    " as a function of "
    "amplitude\",\"required\":[\"amplitude\",\"detection_probability\"]}"
    ,
    /* example: */
    "{\"detection_probability\":[0.0,0.5,0.8,1.0,1.0,1.0,1.0,1.0,1.0,1.0,1.0]"
    ",\"amplitude\":[0.0,1.0,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0,10.0]}"
};

// Details about the device used to acquire the point cloud
const MetaDataItemInfo RDB_RIEGL_DEVICE = {
    /* name        */ "riegl.device",
    /* title       */ "Device Information",
    /* description */ "Details about the device used to acquire the point cloud",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Device Information\",\"type\":\"object\",\"$schema\":\"http://json-s"
    "chema.org/draft-04/schema#\",\"properties\":{\"channel_text\":{\"type\":\"stri"
    "ng\",\"description\":\"Optional channel description (e.g. 'Green Channel' "
    "for multi-channel "
    "devices)\"},\"serial_number\":{\"type\":\"string\",\"description\":\"Device "
    "serial number (e.g. "
    "S2221234)\"},\"device_build\":{\"type\":\"string\",\"description\":\"Device "
    "build variant\"},\"device_type\":{\"type\":\"string\",\"description\":\"Device "
    "type identifier (e.g. "
    "VZ-400i)\"},\"device_name\":{\"type\":\"string\",\"description\":\"Optional "
    "device name (e.g. 'Scanner 1' for multi-scanner systems)\"},\"channel_nu"
    "mber\":{\"exclusiveMinimum\":false,\"type\":\"integer\",\"description\":\"Laser "
    "channel number (not defined or 0: single channel "
    "device)\",\"minimum\":0}},\"description\":\"Details about the device used to"
    " acquire the point cloud\",\"required\":[\"device_type\",\"serial_number\"]}"
    ,
    /* example: */
    "{\"channel_text\":\"\",\"serial_number\":\"S2221234\",\"device_build\":\"\",\"devic"
    "e_type\":\"VZ-400i\",\"device_name\":\"Scanner 1\",\"channel_number\":0}"
};

// Scanner device geometry details
const MetaDataItemInfo RDB_RIEGL_DEVICE_GEOMETRY = {
    /* name        */ "riegl.device_geometry",
    /* title       */ "Device Geometry",
    /* description */ "Scanner device geometry details",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Device Geometry\",\"type\":\"object\",\"$schema\":\"http://json-sche"
    "ma.org/draft-04/schema#\",\"properties\":{\"amu\":{\"properties\":{\"lineCC\":{"
    "\"exclusiveMinimum\":false,\"type\":\"number\",\"description\":\"Line Circle "
    "Count (number of LSBs per full rotation about line axis)\",\"minimum\":0}"
    ",\"frameCC\":{\"exclusiveMinimum\":false,\"type\":\"number\",\"description\":\"Fr"
    "ame Circle Count (number of LSBs per full rotation about frame "
    "axis)\",\"minimum\":0}},\"type\":\"object\",\"description\":\"Angle Measurement "
    "Unit\"},\"secondary\":{\"properties\":{\"ID\":{\"maxItems\":2,\"items\":{\"type\":\""
    "integer\"},\"minItems\":2,\"type\":\"array\",\"description\":\"Structure identif"
    "ier\"},\"content\":{\"items\":{\"type\":\"number\"},\"type\":\"array\",\"description"
    "\":\"Internal calibration "
    "values\"}},\"type\":\"object\",\"description\":\"Additional device geometry "
    "structure (optional)\",\"required\":[\"ID\",\"content\"]},\"primary\":{\"propert"
    "ies\":{\"ID\":{\"maxItems\":2,\"items\":{\"type\":\"integer\"},\"minItems\":2,\"type"
    "\":\"array\",\"description\":\"Structure identifier\"},\"content\":{\"items\":{\"t"
    "ype\":\"number\"},\"type\":\"array\",\"description\":\"Internal calibration "
    "values\"}},\"type\":\"object\",\"description\":\"Primary device geometry "
    "structure "
    "(mandatory)\",\"required\":[\"ID\",\"content\"]}},\"description\":\"Scanner "
    "device geometry details\",\"required\":[\"primary\"]}"
    ,
    /* example: */
    "{\"amu\":{\"lineCC\":124000,\"frameCC\":124000},\"secondary\":{\"ID\":[91,0],\"co"
    "ntent\":[0]},\"primary\":{\"ID\":[4,0],\"content\":[0]}}"
};

// Scanner passive channel device geometry details
const MetaDataItemInfo RDB_RIEGL_DEVICE_GEOMETRY_PASSIVE_CHANNEL = {
    /* name        */ "riegl.device_geometry_passive_channel",
    /* title       */ "Device Geometry Passive Channel",
    /* description */ "Scanner passive channel device geometry details",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Device Geometry Passive Channel\",\"type\":\"object\",\"$schema\":\""
    "http://json-schema.org/draft-04/schema#\",\"properties\":{\"primary\":{\"pro"
    "perties\":{\"ID\":{\"maxItems\":2,\"items\":{\"type\":\"integer\"},\"minItems\":2,\""
    "type\":\"array\",\"description\":\"Structure identifier\"},\"content\":{\"items\""
    ":{\"type\":\"number\"},\"type\":\"array\",\"description\":\"Internal calibration "
    "values\"}},\"type\":\"object\",\"description\":\"Primary device geometry "
    "structure "
    "(mandatory)\",\"required\":[\"ID\",\"content\"]}},\"description\":\"Scanner "
    "passive channel device geometry details\",\"required\":[\"primary\"]}"
    ,
    /* example: */
    "{\"primary\":{\"ID\":[143,0],\"content\":[0]}}"
};

// Limits of the measured values output by the device
const MetaDataItemInfo RDB_RIEGL_DEVICE_OUTPUT_LIMITS = {
    /* name        */ "riegl.device_output_limits",
    /* title       */ "Device Output Limits",
    /* description */ "Limits of the measured values output by the device",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"properties\":{\"re"
    "flectance_maximum\":{\"description\":\"Maximum possible reflectance in dB."
    "\",\"type\":\"number\"},\"background_radiation_maximum\":{\"description\":\"Maxi"
    "mum possible background radiation.\",\"type\":\"number\"},\"mta_zone_count_m"
    "aximum\":{\"description\":\"Maximum number of MTA "
    "zones.\",\"type\":\"number\"},\"amplitude_minimum\":{\"description\":\"Minimum "
    "possible amplitude in "
    "dB.\",\"type\":\"number\"},\"echo_count_maximum\":{\"description\":\"Maximum "
    "number of echoes a laser shot can "
    "have.\",\"type\":\"number\"},\"range_minimum\":{\"description\":\"Minimum "
    "possible range in meters.\",\"type\":\"number\"},\"reflectance_minimum\":{\"de"
    "scription\":\"Minimum possible reflectance in "
    "dB.\",\"type\":\"number\"},\"amplitude_maximum\":{\"description\":\"Maximum "
    "possible amplitude in "
    "dB.\",\"type\":\"number\"},\"deviation_minimum\":{\"description\":\"Minimum "
    "possible pulse shape deviation.\",\"type\":\"number\"},\"background_radiatio"
    "n_minimum\":{\"description\":\"Minimum possible background "
    "radiation.\",\"type\":\"number\"},\"range_maximum\":{\"description\":\"Maximum "
    "possible range in "
    "meters.\",\"type\":\"number\"},\"deviation_maximum\":{\"description\":\"Maximum "
    "possible pulse shape deviation.\",\"type\":\"number\"}},\"title\":\"Device "
    "Output Limits\",\"description\":\"Limits of the measured values output by "
    "the device. The limits depend on the device type, measurement program "
    "and/or scan pattern.\",\"type\":\"object\"}"
    ,
    /* example: */
    "{\"reflectance_minimum\":-100.0,\"amplitude_maximum\":100.0,\"background_ra"
    "diation_maximum\":0,\"deviation_minimum\":-1,\"reflectance_maximum\":100.0,"
    "\"amplitude_minimum\":0.0,\"background_radiation_minimum\":0,\"range_maximu"
    "m\":10000.0,\"mta_zone_count_maximum\":7,\"range_minimum\":2.9,\"deviation_m"
    "aximum\":32767}"
};

// Details about the devices used to acquire the point cloud
const MetaDataItemInfo RDB_RIEGL_DEVICES = {
    /* name        */ "riegl.devices",
    /* title       */ "Devices Information",
    /* description */ "Details about the devices used to acquire the point cloud",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"items\":{\"propert"
    "ies\":{\"channel_text\":{\"type\":\"string\",\"description\":\"Optional channel "
    "description (e.g. 'Green Channel' for multi-channel "
    "devices)\"},\"serial_number\":{\"type\":\"string\",\"description\":\"Device "
    "serial number (e.g. "
    "S2221234)\"},\"device_build\":{\"type\":\"string\",\"description\":\"Device "
    "build variant\"},\"device_type\":{\"type\":\"string\",\"description\":\"Device "
    "type identifier (e.g. "
    "VZ-400i)\"},\"device_name\":{\"type\":\"string\",\"description\":\"Optional "
    "device name (e.g. 'Scanner 1' for multi-scanner systems)\"},\"channel_nu"
    "mber\":{\"exclusiveMinimum\":false,\"type\":\"integer\",\"description\":\"Laser "
    "channel number (not defined or 0: single channel "
    "device)\",\"minimum\":0},\"signed\":{\"type\":\"boolean\",\"description\":\"Flag "
    "that is set when the original 'riegl.device' entry in the source file "
    "was correctly signed.\"}},\"required\":[\"device_type\",\"serial_number\"],\"t"
    "ype\":\"object\"},\"title\":\"Devices Information\",\"description\":\"Details "
    "about the devices used to acquire the point cloud (e.g. when merging "
    "point clouds of different devices)\",\"type\":\"array\"}"
    ,
    /* example: */
    "[{\"channel_text\":\"\",\"serial_number\":\"S2221234\",\"device_build\":\"\",\"devi"
    "ce_type\":\"VZ-400i\",\"device_name\":\"Scanner 1\",\"channel_number\":0,\"signe"
    "d\":false},{\"channel_text\":\"\",\"serial_number\":\"S2222680\",\"device_build\""
    ":\"\",\"device_type\":\"VQ-1560i-DW\",\"device_name\":\"Scanner 2\",\"channel_num"
    "ber\":1,\"signed\":true},{\"channel_text\":\"\",\"serial_number\":\"S2222680\",\"d"
    "evice_build\":\"\",\"device_type\":\"VQ-1560i-DW\",\"device_name\":\"Scanner "
    "3\",\"channel_number\":2,\"signed\":true}]"
};

// Details about echo files
const MetaDataItemInfo RDB_RIEGL_ECHO_INFO = {
    /* name        */ "riegl.echo_info",
    /* title       */ "Echo Information",
    /* description */ "Details about echo files",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Echo Information\",\"type\":\"object\",\"$schema\":\"http://json-sch"
    "ema.org/draft-04/schema#\",\"properties\":{\"echo_file\":{\"properties\":{\"fi"
    "le_uuid\":{\"type\":\"string\",\"description\":\"File's Universally Unique "
    "Identifier (RFC "
    "4122)\"},\"file_extension\":{\"type\":\"string\",\"description\":\"Echo file "
    "extension, without the leading dot\"}},\"required\":[\"file_extension\"],\"t"
    "ype\":\"object\"}},\"description\":\"Details about echo "
    "files\",\"required\":[\"echo_file\"]}"
    ,
    /* example: */
    "{\"echo_file\":{\"file_uuid\":\"26a03615-67c0-4bea-8fe8-c577378fe661\",\"file"
    "_extension\":\"owp\"}}"
};

// Details for exponential decomposition of full waveform data
const MetaDataItemInfo RDB_RIEGL_EXPONENTIAL_DECOMPOSITION = {
    /* name        */ "riegl.exponential_decomposition",
    /* title       */ "Exponential Decomposition",
    /* description */ "Details for exponential decomposition of full waveform data",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"patternProperties\":{\"^[0-9]+$\":{\"$ref\":\"#/definitions/channel\",\"desc"
    "ription\":\"one field per channel, field name is channel "
    "index\"}},\"additionalProperties\":false,\"title\":\"Exponential Decompositi"
    "on\",\"type\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#"
    "\",\"description\":\"Details for exponential decomposition of full "
    "waveform data\",\"definitions\":{\"channel\":{\"properties\":{\"a_lin\":{\"type\""
    ":\"number\",\"maximum\":1,\"minimum\":0,\"exclusiveMaximum\":false,\"exclusiveM"
    "inimum\":false,\"description\":\"relative linear amplitude range [0..1]\"},"
    "\"parameter\":{\"properties\":{\"A\":{\"items\":{\"type\":\"number\"},\"type\":\"arra"
    "y\",\"description\":\"real part of amplitude factor in units of full-scale"
    "\"},\"omega\":{\"items\":{\"type\":\"number\"},\"type\":\"array\",\"description\":\"an"
    "gular frequency in Hz\"},\"gamma\":{\"items\":{\"type\":\"number\"},\"type\":\"arr"
    "ay\",\"description\":\"decay in 1/second\"},\"B\":{\"items\":{\"type\":\"number\"},"
    "\"type\":\"array\",\"description\":\"imaginary part of amplitude factor in "
    "units of full-scale\"}},\"type\":\"object\",\"description\":\"parameters of "
    "the syswave exponential sum\",\"required\":[\"A\",\"B\",\"gamma\",\"omega\"]},\"sc"
    "ale\":{\"type\":\"number\",\"description\":\"amplitude "
    "calibration\"},\"delay\":{\"type\":\"number\",\"description\":\"delay "
    "calibration in "
    "seconds\"}},\"required\":[\"delay\",\"scale\",\"parameter\"],\"type\":\"object\"}}}"
    ,
    /* example: */
    "{\"1\":{\"a_lin\":0.9,\"parameter\":{\"A\":[0.9,0.3,-1.3],\"omega\":[352020896.0"
    ",3647927552.0,-1977987072.0],\"gamma\":[-1094726528.0,-769562752.0,-8480"
    "00064.0],\"B\":[-3.9,0.0,-0.3]},\"scale\":1.0,\"delay\":3.5e-09},\"0\":{\"a_lin"
    "\":0.27,\"parameter\":{\"A\":[0.9772450923919678,0.3354335129261017,-1.3126"
    "78575515747],\"omega\":[352020896.0,3647927552.0,-1977987072.0],\"gamma\":"
    "[-1094726528.0,-769562752.0,-848000064.0],\"B\":[-3.9813032150268555,0.0"
    "8622030913829803,-0.3152860999107361]},\"scale\":1.0,\"delay\":3.783458418"
    "887631e-09}}"
};

// Details for Gaussian decomposition of full waveform data
const MetaDataItemInfo RDB_RIEGL_GAUSSIAN_DECOMPOSITION = {
    /* name        */ "riegl.gaussian_decomposition",
    /* title       */ "Gaussian Decomposition",
    /* description */ "Details for Gaussian decomposition of full waveform data",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Gaussian Decomposition\",\"type\":\"object\",\"$schema\":\"http://js"
    "on-schema.org/draft-04/schema#\",\"properties\":{\"amplitude_db\":{\"items\":"
    "{\"type\":\"number\"},\"type\":\"array\"},\"range_offset_sec_high_power\":{\"item"
    "s\":{\"type\":\"number\"},\"type\":\"array\"},\"amplitude_lsb_low_power\":{\"items"
    "\":{\"type\":\"number\"},\"type\":\"array\"},\"range_offset_sec_low_power\":{\"ite"
    "ms\":{\"type\":\"number\"},\"type\":\"array\"},\"amplitude_lsb_high_power\":{\"ite"
    "ms\":{\"type\":\"number\"},\"type\":\"array\"}},\"description\":\"riegl.gaussian_d"
    "ecomposition contains information relevant for extracting calibrated "
    "amplitudes and calibrated ranges from a Gaussian decomposition of full"
    " waveform data. This information is contained in a table with five "
    "columns. Two columns are to be used as input: amplitude_lsb_low_power "
    "and amplitude_lsb_high_power. The other three columns provide the "
    "outputs. Amplitude_db gives the calibrated amplitude in the optical "
    "regime in decibels. The range offset columns provide additive range "
    "offsets, given in units of seconds, for each channel.\",\"required\":[\"am"
    "plitude_lsb_low_power\",\"amplitude_lsb_high_power\",\"amplitude_db\",\"rang"
    "e_offset_sec_low_power\",\"range_offset_sec_high_power\"]}"
    ,
    /* example: */
    "{\"amplitude_db\":[],\"range_offset_sec_high_power\":[],\"amplitude_lsb_low"
    "_power\":[],\"range_offset_sec_low_power\":[],\"amplitude_lsb_high_power\":"
    "[]}"
};

// Point cloud georeferencing information
const MetaDataItemInfo RDB_RIEGL_GEO_TAG = {
    /* name        */ "riegl.geo_tag",
    /* title       */ "Geo Tag",
    /* description */ "Point cloud georeferencing information",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"properties\":{\"po"
    "se\":{\"maxItems\":4,\"items\":{\"maxItems\":4,\"items\":{\"type\":\"number\",\"desc"
    "ription\":\"columns\"},\"minItems\":4,\"type\":\"array\",\"description\":\"rows\"},"
    "\"minItems\":4,\"description\":\"Coordinate Transformation Matrix to "
    "transform from File Coordinate System to Global Coordinate Reference "
    "System. 4x4 matrix stored as two dimensional array, row major order.\","
    "\"type\":\"array\"},\"crs\":{\"properties\":{\"name\":{\"type\":\"string\",\"descript"
    "ion\":\"Coordinate reference system "
    "name\"},\"epsg\":{\"type\":\"integer\",\"description\":\"EPSG "
    "code\",\"minimum\":0},\"wkt\":{\"type\":\"string\",\"description\":\"\\\"Well-Known "
    "Text\\\" string, OGC WKT dialect (see http://www.opengeospatial.org/stan"
    "dards/wkt-crs)\"}},\"description\":\"Global Coordinate Reference System. "
    "Please note that only 3D Cartesian Coordinate Systems are "
    "allowed.\",\"type\":\"object\"}},\"title\":\"Geo Tag\",\"description\":\"Point "
    "cloud georeferencing information\",\"type\":\"object\"}"
    ,
    /* example: */
    "{\"pose\":[[-0.269827776749716,-0.723017716139738,0.635954678449952,4063"
    "882.500831],[0.962908599449764,-0.20260517250352,0.178208229833847,113"
    "8787.607461],[0.0,0.660451759194338,0.7508684796801,4766084.550196],[0"
    ".0,0.0,0.0,1.0]],\"crs\":{\"name\":\"WGS84 "
    "Geocentric\",\"epsg\":4978,\"wkt\":\"GEOCCS[\\\"WGS84 Geocentric\\\",DATUM[\\\"WGS"
    "84\\\",SPHEROID[\\\"WGS84\\\",6378137.000,298.257223563,AUTHORITY[\\\"EPSG\\\",\\"
    "\"7030\\\"]],AUTHORITY[\\\"EPSG\\\",\\\"6326\\\"]],PRIMEM[\\\"Greenwich\\\",0.0000000"
    "000000000,AUTHORITY[\\\"EPSG\\\",\\\"8901\\\"]],UNIT[\\\"Meter\\\",1.0000000000000"
    "0000000,AUTHORITY[\\\"EPSG\\\",\\\"9001\\\"]],AXIS[\\\"X\\\",OTHER],AXIS[\\\"Y\\\",EAS"
    "T],AXIS[\\\"Z\\\",NORTH],AUTHORITY[\\\"EPSG\\\",\\\"4978\\\"]]\"}}"
};

// Geometric scale factor applied to point coordinates
const MetaDataItemInfo RDB_RIEGL_GEOMETRIC_SCALE_FACTOR = {
    /* name        */ "riegl.geometric_scale_factor",
    /* title       */ "Geometric Scale Factor",
    /* description */ "Geometric scale factor applied to point coordinates",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"exclusiveMinimum"
    "\":true,\"type\":\"number\",\"description\":\"Geometric scale factor applied "
    "to point coordinates\",\"minimum\":0}"
    ,
    /* example: */
    "1.0"
};

// Parameters used for georeferencing of the point cloud
const MetaDataItemInfo RDB_RIEGL_GEOREFERENCING_PARAMETERS = {
    /* name        */ "riegl.georeferencing_parameters",
    /* title       */ "Georeferencing Parameters",
    /* description */ "Parameters used for georeferencing of the point cloud",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"properties\":{\"so"
    "cs_to_rocs_matrix\":{\"maxItems\":4,\"items\":{\"maxItems\":4,\"items\":{\"type\""
    ":\"number\",\"description\":\"columns\"},\"minItems\":4,\"type\":\"array\",\"descri"
    "ption\":\"rows\"},\"minItems\":4,\"description\":\"Coordinate Transformation "
    "Matrix to transform from Scanner's Own Coordinate System to Record "
    "Coordinate System. 4x4 matrix stored as two dimensional array, row "
    "major order.\",\"type\":\"array\"},\"socs_to_body_matrix\":{\"maxItems\":4,\"ite"
    "ms\":{\"maxItems\":4,\"items\":{\"type\":\"number\",\"description\":\"columns\"},\"m"
    "inItems\":4,\"type\":\"array\",\"description\":\"rows\"},\"minItems\":4,\"descript"
    "ion\":\"Coordinate Transformation Matrix to transform from Scanner's Own"
    " Coordinate System to Body Coordinate System. 4x4 matrix stored as two"
    " dimensional array, row major order.\",\"type\":\"array\"},\"body_coordinate"
    "_system_type\":{\"description\":\"BODY coordinate frame (NED: "
    "North-East-Down, ENU: East-North-Up), default: NED\",\"type\":\"string\",\"e"
    "num\":[\"NED\",\"ENU\"]},\"trajectory_offsets\":{\"properties\":{\"offset_roll\":"
    "{\"description\":\"[deg]\",\"type\":\"number\"},\"offset_yaw\":{\"description\":\"["
    "deg]\",\"type\":\"number\"},\"offset_pitch\":{\"description\":\"[deg]\",\"type\":\"n"
    "umber\"},\"offset_height\":{\"description\":\"[m]\",\"type\":\"number\"},\"offset_"
    "east\":{\"description\":\"[m]\",\"type\":\"number\"},\"version\":{\"description\":\""
    "Meaning of offset values and how to apply them; version 0: "
    "Rz(yaw+offset_yaw)*Ry(pitch+offset_pitch)*Rx(roll+offset_roll), "
    "version 1: Rz(yaw)*Ry(pitch)*Rx(roll) * Rz(offset_yaw)*Ry(offset_pitch"
    ")*Rx(offset_roll)\",\"type\":\"integer\"},\"offset_north\":{\"description\":\"[m"
    "]\",\"type\":\"number\"},\"offset_time\":{\"description\":\"[s]\",\"type\":\"number\""
    "}},\"description\":\"Correction offsets applied to the trajectory data\",\""
    "type\":\"object\"},\"trajectory_file\":{\"properties\":{\"file_uuid\":{\"type\":\""
    "string\",\"description\":\"File's Universally Unique Identifier (RFC "
    "4122)\"},\"file_extension\":{\"type\":\"string\",\"description\":\"Trajectory "
    "file extension, without the leading dot\"}},\"description\":\"Trajectory "
    "data used for georeferencing of the point cloud\",\"type\":\"object\",\"requ"
    "ired\":[\"file_extension\"]}},\"title\":\"Georeferencing "
    "Parameters\",\"description\":\"Parameters used for georeferencing of the "
    "point cloud\",\"type\":\"object\"}"
    ,
    /* example: */
    "{\"socs_to_body_matrix\":[[-0.269827776749716,-0.723017716139738,0.63595"
    "4678449952,0.0],[0.962908599449764,-0.20260517250352,0.178208229833847"
    ",0.0],[0.0,0.660451759194338,0.7508684796801,0.0],[0.0,0.0,0.0,1.0]],\""
    "body_coordinate_system_type\":\"NED\",\"trajectory_offsets\":{\"offset_roll\""
    ":0.03,\"offset_yaw\":-0.45,\"offset_pitch\":0.01,\"offset_height\":-0.2,\"off"
    "set_east\":0.15,\"version\":0,\"offset_north\":0.07,\"offset_time\":18.007},\""
    "trajectory_file\":{\"file_uuid\":\"93a03615-66c0-4bea-8ff8-c577378fe660\",\""
    "file_extension\":\"pofx\"}}"
};

// Map of item names
const MetaDataItemInfo RDB_RIEGL_ITEM_NAMES = {
    /* name        */ "riegl.item_names",
    /* title       */ "Item Names",
    /* description */ "Map of item names",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"patternProperties\":{\"^-?[0-9]+$\":{\"type\":\"string\",\"description\":\"One"
    " field per item, field name is item id, field value is item "
    "name\"}},\"additionalProperties\":false,\"title\":\"Item Names\",\"type\":\"obje"
    "ct\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"description\":"
    "\"Map of item names\"}"
    ,
    /* example: */
    "{\"1\":\"Name of item with id 1\",\"47\":\"Name of item with id "
    "47\",\"-1\":\"Name of item with id -1\",\"2\":\"Name of item with id 2\"}"
};

// License keys for software features
const MetaDataItemInfo RDB_RIEGL_LICENSES = {
    /* name        */ "riegl.licenses",
    /* title       */ "Software License Keys",
    /* description */ "License keys for software features",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"patternProperties\":{\"^.*$\":{\"items\":{\"type\":\"string\",\"description\":\""
    "License key (example: '46AE032A - 39882AC4 - 9EC0A184 - "
    "6F163D73')\"},\"minItems\":1,\"type\":\"array\",\"description\":\"Each field of "
    "the object represents a feature and holds a list of license keys, "
    "where the field name is the feature "
    "name.\"}},\"additionalProperties\":false,\"title\":\"Software License Keys\","
    "\"type\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"d"
    "escription\":\"License keys for software features\"}"
    ,
    /* example: */
    "{\"Full Waveform Analysis Topography\":[\"0FD5FF07 - 011A1255 - 9F76CACA "
    "- 8D2ED557\"],\"Georeferencing\":[\"46AE032A - 39882AC4 - 9EC0A184 - "
    "6F163D73\"],\"MTA resolution\":[\"468E020A - 39A922E4 - B681A184 - "
    "673E3D72\"],\"Full Waveform Analysis Topography with GPU "
    "support\":[\"8AB44126 - 23B92250 - 16E2689F - 34EF7E7B\"]}"
};

// Parameters for MTA processing
const MetaDataItemInfo RDB_RIEGL_MTA_SETTINGS = {
    /* name        */ "riegl.mta_settings",
    /* title       */ "MTA Settings",
    /* description */ "Parameters for MTA processing",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"properties\":{\"zo"
    "ne_width\":{\"description\":\"Width of a MTA zone in meter.\",\"type\":\"numbe"
    "r\",\"minimum\":0},\"zone_count\":{\"maximum\":255,\"description\":\"Maximum "
    "number of MTA zones.\",\"type\":\"integer\",\"minimum\":0},\"modulation_depth\""
    ":{\"description\":\"Depth of pulse position modulation in "
    "meter.\",\"type\":\"number\",\"minimum\":0}},\"title\":\"MTA "
    "Settings\",\"description\":\"Parameters for MTA "
    "processing\",\"type\":\"object\"}"
    ,
    /* example: */
    "{\"zone_width\":149.896225,\"zone_count\":23,\"modulation_depth\":9.368514}"
};

// Lookup table for range correction based on raw range
const MetaDataItemInfo RDB_RIEGL_NEAR_RANGE_CORRECTION = {
    /* name        */ "riegl.near_range_correction",
    /* title       */ "Near Range Correction Table",
    /* description */ "Lookup table for range correction based on raw range",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Near Range Correction Table\",\"type\":\"object\",\"$schema\":\"http"
    "://json-schema.org/draft-04/schema#\",\"properties\":{\"delta\":{\"type\":\"nu"
    "mber\",\"description\":\"Delta between table entries [m], first entry is "
    "at range = 0 m\"},\"content\":{\"maxItems\":2000,\"items\":{\"type\":\"number\"},"
    "\"minItems\":1,\"type\":\"array\",\"description\":\"Correction value [m] to be "
    "added to the raw range\"}},\"description\":\"Lookup table for range "
    "correction based on raw range\",\"required\":[\"delta\",\"content\"]}"
    ,
    /* example: */
    "{\"delta\":0.512,\"content\":[0.0]}"
};

// Standard deviation for range and amplitude as a function of amplitude
const MetaDataItemInfo RDB_RIEGL_NOISE_ESTIMATES = {
    /* name        */ "riegl.noise_estimates",
    /* title       */ "Noise Estimates",
    /* description */ "Standard deviation for range and amplitude as a function of amplitude",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Noise Estimates\",\"type\":\"object\",\"$schema\":\"http://json-sche"
    "ma.org/draft-04/schema#\",\"properties\":{\"amplitude\":{\"items\":{\"type\":\"n"
    "umber\"},\"description\":\"Amplitude [dB]\",\"type\":\"array\"},\"range_sigma\":{"
    "\"items\":{\"type\":\"number\"},\"description\":\"Sigma range [m]\",\"type\":\"arra"
    "y\"},\"amplitude_sigma\":{\"items\":{\"type\":\"number\"},\"description\":\"Sigma "
    "amplitude [dB]\",\"type\":\"array\"}},\"description\":\"Standard deviation for"
    " range and amplitude as a function of "
    "amplitude\",\"required\":[\"amplitude\",\"range_sigma\",\"amplitude_sigma\"]}"
    ,
    /* example: */
    "{\"amplitude\":[0.0,1.0,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0,10.0],\"range_sig"
    "ma\":[0.065,0.056,0.046,0.038,0.032,0.027,0.024,0.021,0.018,0.016,0.014"
    "],\"amplitude_sigma\":[0.988,0.988,0.875,0.774,0.686,0.608,0.54,0.482,0."
    "432,0.39,0.354]}"
};

// Notch filter parameters for window glass echoes
const MetaDataItemInfo RDB_RIEGL_NOTCH_FILTER = {
    /* name        */ "riegl.notch_filter",
    /* title       */ "Notch Filter",
    /* description */ "Notch filter parameters for window glass echoes",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Notch Filter\",\"type\":\"object\",\"$schema\":\"http://json-schema."
    "org/draft-04/schema#\",\"properties\":{\"range_maximum\":{\"description\":\"Ma"
    "ximum range "
    "[m]\",\"type\":\"number\"},\"amplitude_maximum\":{\"description\":\"Maximum "
    "amplitude [dB]\",\"type\":\"number\",\"minimum\":0},\"range_minimum\":{\"descrip"
    "tion\":\"Minimum range [m]\",\"type\":\"number\"}},\"description\":\"Notch "
    "filter parameters for window glass echoes\",\"required\":[\"range_minimum\""
    ",\"range_maximum\",\"amplitude_maximum\"]}"
    ,
    /* example: */
    "{\"range_maximum\":0.2,\"amplitude_maximum\":18.0,\"range_minimum\":-0.5}"
};

// Details about the pixels contained in the file
const MetaDataItemInfo RDB_RIEGL_PIXEL_INFO = {
    /* name        */ "riegl.pixel_info",
    /* title       */ "Pixel Information",
    /* description */ "Details about the pixels contained in the file",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Pixel Information\",\"type\":\"object\",\"$schema\":\"http://json-sc"
    "hema.org/draft-04/schema#\",\"properties\":{\"size\":{\"$ref\":\"#/definitions"
    "/pixel_size\",\"description\":\"Size of pixels in file coordinate system.\""
    "},\"size_llcs\":{\"$ref\":\"#/definitions/pixel_size\",\"description\":\"Size "
    "of pixels in a locally levelled cartesian coordinate system (xy). This"
    " is only used for pixels based on a map "
    "projection.\"}},\"required\":[\"size\"],\"description\":\"Details about the "
    "pixels contained in the file\",\"definitions\":{\"pixel_size\":{\"maxItems\":"
    "2,\"items\":{\"description\":\"Length of pixel edge "
    "[m].\",\"type\":\"number\",\"minimum\":0},\"minItems\":2,\"description\":\"Size of"
    " pixels.\",\"type\":\"array\"}}}"
    ,
    /* example: */
    "{\"size\":[0.5971642834779395,0.5971642834779395],\"size_llcs\":[0.5156575"
    "252891171,0.5130835356683303]}"
};

// Details about the plane patch matching process
const MetaDataItemInfo RDB_RIEGL_PLANE_PATCH_MATCHING = {
    /* name        */ "riegl.plane_patch_matching",
    /* title       */ "Plane Patch Matching",
    /* description */ "Details about the plane patch matching process",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Plane Patch Matching\",\"type\":\"object\",\"$schema\":\"http://json"
    "-schema.org/draft-04/schema#\",\"properties\":{\"plane_patch_file_two\":{\"$"
    "ref\":\"#/definitions/file_reference\",\"description\":\"Reference to the "
    "plane patch file two\"},\"plane_patch_file_one\":{\"$ref\":\"#/definitions/f"
    "ile_reference\",\"description\":\"Reference to the plane patch file one\"}}"
    ",\"required\":[\"plane_patch_file_one\",\"plane_patch_file_two\"],\"descripti"
    "on\":\"Details about the plane patch matching process\",\"definitions\":{\"f"
    "ile_reference\":{\"properties\":{\"file_uuid\":{\"type\":\"string\",\"descriptio"
    "n\":\"Plane patch file's Universally Unique Identifier (RFC "
    "4122)\"},\"file_path\":{\"type\":\"string\",\"description\":\"Path of the plane "
    "patch file relative to the match file\"}},\"description\":\"Reference to a"
    " plane patch "
    "file\",\"type\":\"object\",\"required\":[\"file_uuid\",\"file_path\"]}}}"
    ,
    /* example: */
    "{\"plane_patch_file_two\":{\"file_uuid\":\"fa47d509-a64e-49ce-8b14-ff3130fb"
    "efa9\",\"file_path\":\"project.ptch\"},\"plane_patch_file_one\":{\"file_uuid\":"
    "\"810f5d57-eccf-49ed-b07a-0cdd109b4213\",\"file_path\":\"Record009_Line001/"
    "191025_121410_Scanner_1.ptch\"}}"
};

// Statistics about plane patches found by plane patch extractor
const MetaDataItemInfo RDB_RIEGL_PLANE_PATCH_STATISTICS = {
    /* name        */ "riegl.plane_patch_statistics",
    /* title       */ "Plane Patch Statistics",
    /* description */ "Statistics about plane patches found by plane patch extractor",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"properties\":{\"to"
    "tal_horizontal_area\":{\"description\":\"sum of all plane patch areas "
    "projected to horizontal plane "
    "[m\\u00b2]\",\"type\":\"number\"},\"total_area\":{\"description\":\"sum of all "
    "plane patch areas [m\\u00b2]\",\"type\":\"number\"}},\"title\":\"Plane Patch "
    "Statistics\",\"description\":\"Statistics about plane patches found by "
    "plane patch extractor\",\"type\":\"object\"}"
    ,
    /* example: */
    "{\"total_horizontal_area\":13954.601,\"total_area\":14007.965}"
};

// Settings and classes for plane slope classification
const MetaDataItemInfo RDB_RIEGL_PLANE_SLOPE_CLASS_INFO = {
    /* name        */ "riegl.plane_slope_class_info",
    /* title       */ "Plane Slope Class Info",
    /* description */ "Settings and classes for plane slope classification",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"definitions\":{\"method-1\":{\"properties\":{\"maximum_inclination_angle_h"
    "orizontal\":{\"maximum\":360.0,\"description\":\"maximum inclination angle "
    "of horizontal plane patches [deg]\",\"type\":\"number\",\"minimum\":-360.0},\""
    "plane_classification_method\":{\"maximum\":1,\"description\":\"method ID "
    "(=1)\",\"type\":\"integer\",\"minimum\":1}},\"description\":\"Classification "
    "method 1\",\"type\":\"object\",\"required\":[\"plane_classification_method\",\"m"
    "aximum_inclination_angle_horizontal\"]},\"method-2\":{\"properties\":{\"slop"
    "ing_plane_classes_minimum_angle\":{\"maximum\":360.0,\"description\":\"minim"
    "um inclination angle of sloping plane patches [deg]\",\"type\":\"number\",\""
    "minimum\":-360.0},\"sloping_plane_classes_maximum_angle\":{\"maximum\":360."
    "0,\"description\":\"maximum inclination angle of sloping plane patches [d"
    "eg]\",\"type\":\"number\",\"minimum\":-360.0},\"plane_classification_method\":{"
    "\"maximum\":2,\"description\":\"method ID "
    "(=2)\",\"type\":\"integer\",\"minimum\":2}},\"description\":\"Classification "
    "method 2\",\"type\":\"object\",\"required\":[\"plane_classification_method\",\"s"
    "loping_plane_classes_minimum_angle\",\"sloping_plane_classes_maximum_ang"
    "le\"]}},\"type\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/sche"
    "ma#\",\"properties\":{\"classes\":{\"patternProperties\":{\"^[0-9]+$\":{\"type\":"
    "\"string\",\"description\":\"one field per class, field name is class "
    "number, field value is class "
    "name\"}},\"additionalProperties\":false,\"description\":\"Class definition t"
    "able\",\"type\":\"object\"},\"settings\":{\"oneOf\":[{\"$ref\":\"#/definitions/met"
    "hod-1\"},{\"$ref\":\"#/definitions/method-2\"}],\"description\":\"Classificati"
    "on settings, details see documentation of rdbplanes\",\"type\":\"object\"}}"
    ",\"required\":[\"settings\",\"classes\"],\"description\":\"Settings and classes"
    " for plane slope classification\",\"title\":\"Plane Slope Class Info\"}"
    ,
    /* example: */
    "{\"classes\":{\"11\":\"sloping, pointing down and south\",\"10\":\"sloping, "
    "pointing down and east\",\"13\":\"sloping, pointing down and "
    "west\",\"14\":\"horizontal, pointing down\",\"2\":\"sloping, pointing up and "
    "east\",\"6\":\"vertical, pointing east\",\"12\":\"sloping, pointing down and "
    "north\",\"3\":\"sloping, pointing up and south\",\"5\":\"sloping, pointing up "
    "and west\",\"7\":\"vertical, pointing south\",\"9\":\"vertical, pointing "
    "west\",\"1\":\"horizontal, pointing up\",\"4\":\"sloping, pointing up and "
    "north\",\"8\":\"vertical, pointing north\"},\"settings\":{\"sloping_plane_clas"
    "ses_minimum_angle\":10.0,\"sloping_plane_classes_maximum_angle\":70.0,\"pl"
    "ane_classification_method\":2}}"
};

// Grouping and sorting of point attributes for visualization purposes
const MetaDataItemInfo RDB_RIEGL_POINT_ATTRIBUTE_GROUPS = {
    /* name        */ "riegl.point_attribute_groups",
    /* title       */ "Point Attribute Groups",
    /* description */ "Grouping and sorting of point attributes for visualization purposes",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"patternProperties\":{\"^.*$\":{\"items\":{\"type\":\"string\",\"description\":\""
    "Point attribute full name or name pattern (perl regular expression "
    "syntax)\"},\"minItems\":1,\"type\":\"array\",\"description\":\"Each field of the"
    " object represents a point attribute group and holds a list of point "
    "attributes, where the field name is the group "
    "name.\"}},\"additionalProperties\":false,\"title\":\"Point Attribute Groups\""
    ",\"type\":\"object\",\"$schema\":\"http://json-schema.org/draft-04/schema#\",\""
    "description\":\"Grouping and sorting of point attributes for "
    "visualization purposes\"}"
    ,
    /* example: */
    "{\"Coordinates/Vectors\":[\"riegl.xyz\",\"riegl.range\",\"riegl.theta\",\"riegl"
    ".phi\"],\"Other Attributes\":[\"riegl.selected\",\"riegl.visible\"],\"Primary "
    "Attributes\":[\"riegl.reflectance\",\"riegl.amplitude\",\"riegl.deviation\"],"
    "\"Secondary Attributes\":[\"riegl.mirror_facet\",\"riegl.waveform_available"
    "\"],\"Time\":[\"riegl.timestamp\"]}"
};

// Details about point cloud files
const MetaDataItemInfo RDB_RIEGL_POINTCLOUD_INFO = {
    /* name        */ "riegl.pointcloud_info",
    /* title       */ "Point Cloud Information",
    /* description */ "Details about point cloud files",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"properties\":{\"co"
    "mments\":{\"type\":\"string\",\"description\":\"Comments\"},\"project\":{\"type\":\""
    "string\",\"description\":\"Project "
    "name\"},\"field_of_application\":{\"type\":\"string\",\"description\":\"Field of"
    " application\",\"enum\":[\"unknown\",\"SLS\",\"TLS\",\"KLS\",\"MLS\",\"ULS\",\"ALS\",\"B"
    "LS\",\"ILS\"]}},\"title\":\"Point Cloud Information\",\"description\":\"Details "
    "about point cloud files\",\"type\":\"object\"}"
    ,
    /* example: */
    "{\"comments\":\"Line 3\",\"project\":\"Campaign "
    "4\",\"field_of_application\":\"ALS\"}"
};

// Estimated position and orientation information
const MetaDataItemInfo RDB_RIEGL_POSE_ESTIMATION = {
    /* name        */ "riegl.pose_estimation",
    /* title       */ "Pose Estimation",
    /* description */ "Estimated position and orientation information",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Pose Estimation\",\"type\":\"object\",\"$schema\":\"http://json-sche"
    "ma.org/draft-04/schema#\",\"properties\":{\"barometric_height_amsl\":{\"type"
    "\":\"number\",\"description\":\"Altitude determined based on the atmospheric"
    " pressure according to the standard atmosphere laws [m].\"},\"position\":"
    "{\"properties\":{\"vertical_accuracy\":{\"exclusiveMinimum\":true,\"type\":\"nu"
    "mber\",\"description\":\"Vertical accuracy [m]\",\"minimum\":0},\"coordinate_1"
    "\":{\"type\":\"number\",\"description\":\"Coordinate 1 as defined by axis 1 of"
    " the specified CRS (e.g., X, "
    "Latitude)\"},\"coordinate_2\":{\"type\":\"number\",\"description\":\"Coordinate "
    "2 as defined by axis 2 of the specified CRS (e.g., Y, "
    "Longitude)\"},\"coordinate_3\":{\"type\":\"number\",\"description\":\"Coordinate"
    " 3 as defined by axis 3 of the specified CRS (e.g., Z, Altitude)\"},\"cr"
    "s\":{\"properties\":{\"wkt\":{\"type\":\"string\",\"description\":\"\\\"Well-Known "
    "Text\\\" string, OGC WKT dialect (see http://www.opengeospatial.org/stan"
    "dards/wkt-crs)\"},\"epsg\":{\"type\":\"integer\",\"description\":\"EPSG "
    "code\",\"minimum\":0}},\"description\":\"Global Coordinate Reference System\""
    ",\"type\":\"object\",\"required\":[\"epsg\"]},\"horizontal_accuracy\":{\"exclusiv"
    "eMinimum\":true,\"type\":\"number\",\"description\":\"Horizontal accuracy "
    "[m]\",\"minimum\":0}},\"type\":\"object\",\"description\":\"Position coordinates"
    " and position accuracy values as measured by GNSS in the specified "
    "Coordinate Reference System (CRS)\",\"required\":[\"coordinate_1\",\"coordin"
    "ate_2\",\"coordinate_3\",\"horizontal_accuracy\",\"vertical_accuracy\",\"crs\"]"
    "},\"orientation\":{\"properties\":{\"pitch\":{\"maximum\":360,\"type\":\"number\","
    "\"description\":\"Pitch angle about scanner Y-axis [deg]\",\"minimum\":-360}"
    ",\"yaw\":{\"maximum\":360,\"type\":\"number\",\"description\":\"Yaw angle about "
    "scanner Z-axis [deg]\",\"minimum\":-360},\"roll\":{\"maximum\":360,\"type\":\"nu"
    "mber\",\"description\":\"Roll angle about scanner X-axis [deg]\",\"minimum\":"
    "-360},\"pitch_accuracy\":{\"exclusiveMinimum\":true,\"type\":\"number\",\"descr"
    "iption\":\"Pitch angle accuracy [deg]\",\"minimum\":0},\"roll_accuracy\":{\"ex"
    "clusiveMinimum\":true,\"type\":\"number\",\"description\":\"Roll angle "
    "accuracy [deg]\",\"minimum\":0},\"yaw_accuracy\":{\"exclusiveMinimum\":true,\""
    "type\":\"number\",\"description\":\"Yaw angle accuracy "
    "[deg]\",\"minimum\":0}},\"type\":\"object\",\"description\":\"Orientation values"
    " and orientation accuracies, measured with IMU or inclination sensors."
    "\",\"required\":[\"roll\",\"pitch\",\"yaw\",\"roll_accuracy\",\"pitch_accuracy\",\"y"
    "aw_accuracy\"]}},\"description\":\"Estimated position and orientation "
    "information as measured by GNSS, IMU or inclination "
    "sensors\",\"required\":[\"orientation\"]}"
    ,
    /* example: */
    "{\"barometric_height_amsl\":386.7457796227932,\"position\":{\"vertical_accu"
    "racy\":1.3314999341964722,\"coordinate_1\":48.655799473,\"coordinate_2\":15"
    ".645033406,\"coordinate_3\":362.7124938964844,\"crs\":{\"wkt\":\"GEOGCS[\\\"WGS"
    "84 / Geographic\\\",DATUM[\\\"WGS84\\\",SPHEROID[\\\"WGS84\\\",6378137.000,298.2"
    "57223563,AUTHORITY[\\\"EPSG\\\",\\\"7030\\\"]],AUTHORITY[\\\"EPSG\\\",\\\"6326\\\"]],P"
    "RIMEM[\\\"Greenwich\\\",0.0000000000000000,AUTHORITY[\\\"EPSG\\\",\\\"8901\\\"]],U"
    "NIT[\\\"Degree\\\",0.01745329251994329547,AUTHORITY[\\\"EPSG\\\",\\\"9102\\\"]],AX"
    "IS[\\\"Latitude\\\",NORTH],AXIS[\\\"Longitude\\\",EAST],AUTHORITY[\\\"EPSG\\\",\\\"4"
    "979\\\"]]\",\"epsg\":4979},\"horizontal_accuracy\":0.810699999332428},\"orient"
    "ation\":{\"pitch\":1.509153024827064,\"yaw\":101.87293630292045,\"roll\":3.14"
    "743073066123,\"pitch_accuracy\":0.009433783936875745,\"roll_accuracy\":0.0"
    "09433783936875745,\"yaw_accuracy\":1.0094337839368757}}"
};

// Details on position and orientation sensors
const MetaDataItemInfo RDB_RIEGL_POSE_SENSORS = {
    /* name        */ "riegl.pose_sensors",
    /* title       */ "Pose Sensors",
    /* description */ "Details on position and orientation sensors",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Pose Sensors\",\"type\":\"object\",\"$schema\":\"http://json-schema."
    "org/draft-04/schema#\",\"properties\":{\"gyroscope\":{\"properties\":{\"origin"
    "\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Sensor origin in SOCS "
    "[m] at frame angle = "
    "0\"},\"x_axis\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Sensitive X"
    " axis of sensor at frame angle = "
    "0\"},\"z_axis\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Sensitive Z"
    " axis of sensor at frame angle = "
    "0\"},\"y_axis\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Sensitive Y"
    " axis of sensor at frame angle = "
    "0\"},\"offset\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Value to be"
    " subtracted from raw measurement "
    "values\"},\"unit\":{\"exclusiveMinimum\":true,\"description\":\"Unit of raw "
    "data and calibration values, 1 LSB in rad/s\",\"type\":\"number\",\"minimum\""
    ":0},\"relative_nonlinearity\":{\"$ref\":\"#/definitions/vector\",\"descriptio"
    "n\":\"Relative nonlinearity, dimensionless\"}},\"description\":\"Gyroscope d"
    "etails\",\"type\":\"object\",\"required\":[\"unit\",\"x_axis\",\"y_axis\",\"z_axis\","
    "\"offset\",\"origin\",\"relative_nonlinearity\"]},\"accelerometer\":{\"properti"
    "es\":{\"origin\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Sensor "
    "origin in SOCS [m] at frame angle = "
    "0\"},\"x_axis\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Sensitive X"
    " axis of sensor at frame angle = "
    "0\"},\"z_axis\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Sensitive Z"
    " axis of sensor at frame angle = "
    "0\"},\"y_axis\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Sensitive Y"
    " axis of sensor at frame angle = "
    "0\"},\"offset\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Value to be"
    " subtracted from raw measurement "
    "values\"},\"unit\":{\"exclusiveMinimum\":true,\"description\":\"Unit of raw "
    "data and calibration values, 1 LSB in 9.81 m/s\\u00b2\",\"type\":\"number\","
    "\"minimum\":0},\"relative_nonlinearity\":{\"$ref\":\"#/definitions/vector\",\"d"
    "escription\":\"Relative nonlinearity, "
    "dimensionless\"}},\"description\":\"Accelerometer details\",\"type\":\"object\""
    ",\"required\":[\"unit\",\"x_axis\",\"y_axis\",\"z_axis\",\"offset\",\"origin\",\"rela"
    "tive_nonlinearity\"]},\"magnetic_field_sensor\":{\"properties\":{\"x_axis\":{"
    "\"$ref\":\"#/definitions/vector\",\"description\":\"Sensitive X axis of "
    "sensor at frame angle = "
    "0\"},\"z_axis\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Sensitive Z"
    " axis of sensor at frame angle = "
    "0\"},\"y_axis\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Sensitive Y"
    " axis of sensor at frame angle = "
    "0\"},\"fixed\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Distortion "
    "of magnetic field caused by non-rotating scanner "
    "part\"},\"offset\":{\"$ref\":\"#/definitions/vector\",\"description\":\"Value to"
    " be subtracted from raw measurement "
    "values\"},\"unit\":{\"exclusiveMinimum\":true,\"description\":\"Unit of raw "
    "data and calibration values, 1 LSB in nT\",\"type\":\"number\",\"minimum\":0}"
    ",\"relative_nonlinearity\":{\"$ref\":\"#/definitions/vector\",\"description\":"
    "\"Relative nonlinearity, dimensionless\"}},\"description\":\"Magnetic Field"
    " Sensor details\",\"type\":\"object\",\"required\":[\"unit\",\"x_axis\",\"y_axis\","
    "\"z_axis\",\"offset\",\"fixed\",\"relative_nonlinearity\"]}},\"required\":[\"gyro"
    "scope\",\"accelerometer\",\"magnetic_field_sensor\"],\"description\":\"Details"
    " on position and orientation sensors\",\"definitions\":{\"vector\":{\"maxIte"
    "ms\":3,\"items\":{\"description\":\"Index 0=X, 1=Y, 2=Z "
    "component\",\"type\":\"number\"},\"minItems\":3,\"type\":\"array\"}}}"
    ,
    /* example: */
    "{\"gyroscope\":{\"origin\":[0.026900000870227814,-0.03999999910593033,-0.0"
    "8950000256299973],\"x_axis\":[-121.195556640625,0.8219714164733887,0.231"
    "3031703233719],\"z_axis\":[0.555869996547699,119.22135162353516,0.467585"
    "027217865],\"y_axis\":[-0.440765917301178,-0.7897399663925171,119.589477"
    "5390625],\"offset\":[-50.92609786987305,146.15643310546875,62.4327278137"
    "207],\"unit\":0.00014544410805683583,\"relative_nonlinearity\":[2.88817631"
    "1444113e-07,1.06274164579645e-07,-1.7186295080634935e-39]},\"accelerome"
    "ter\":{\"origin\":[0.026900000870227814,-0.03999999910593033,-0.089500002"
    "56299973],\"x_axis\":[-15008.123046875,56.956390380859375,-60.5175666809"
    "082],\"z_axis\":[1.639882206916809,15166.744140625,-116.99742889404297],"
    "\"y_axis\":[-7.027288913726807,-44.12333679199219,14952.3701171875],\"off"
    "set\":[-733.3636474609375,58.969032287597656,1060.2550048828125],\"unit\""
    ":6.666666740784422e-05,\"relative_nonlinearity\":[0.0,0.0,0.0]},\"magneti"
    "c_field_sensor\":{\"x_axis\":[-0.011162743903696537,-2.315962774446234e-0"
    "5,0.00016818844596855342],\"z_axis\":[0.00041987866279669106,7.876977906"
    "562388e-05,0.011407104320824146],\"y_axis\":[0.00027888521435670555,-0.0"
    "11427424848079681,-5.204829722060822e-05],\"fixed\":[-1576.010498046875,"
    "1596.081787109375,0.0],\"offset\":[-23812.052734375,5606.57666015625,249"
    "3.28125],\"unit\":91.74311828613281,\"relative_nonlinearity\":[0.0,0.0,0.0"
    "]}}"
};

// Laser pulse position modulation used for MTA resolution
const MetaDataItemInfo RDB_RIEGL_PULSE_POSITION_MODULATION = {
    /* name        */ "riegl.pulse_position_modulation",
    /* title       */ "Pulse Position Modulation",
    /* description */ "Laser pulse position modulation used for MTA resolution",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Pulse Position Modulation\",\"type\":\"object\",\"$schema\":\"http:/"
    "/json-schema.org/draft-04/schema#\",\"properties\":{\"num_mod_ampl\":{\"maxi"
    "mum\":255,\"description\":\"Number of different modulation amplitudes (2: "
    "binary modulation)\",\"type\":\"integer\",\"minimum\":0},\"phase_step\":{\"maxim"
    "um\":255,\"description\":\"Step width in phase of modulation code from "
    "line to line\",\"type\":\"integer\",\"minimum\":0},\"code_phase_mode\":{\"maximu"
    "m\":255,\"description\":\"0: no synchronization, 1: toggle between 2 "
    "phases, 2: increment with phase_increment\",\"type\":\"integer\",\"minimum\":"
    "0},\"length\":{\"maximum\":255,\"description\":\"Length of code\",\"type\":\"inte"
    "ger\",\"minimum\":0},\"pulse_interval\":{\"items\":{\"type\":\"number\",\"minimum\""
    ":0},\"description\":\"Explicit table of the pulse position modulation "
    "used for MTA resolution. Table gives times between successive laser "
    "pulses in seconds.\",\"type\":\"array\"}},\"description\":\"Laser pulse "
    "position modulation used for MTA "
    "resolution\",\"required\":[\"length\",\"num_mod_ampl\",\"pulse_interval\"]}"
    ,
    /* example: */
    "{\"num_mod_ampl\":2,\"phase_step\":5,\"code_phase_mode\":2,\"length\":31,\"puls"
    "e_interval\":[2.759375e-06,2.759375e-06,2.759375e-06,2.759375e-06,2.821"
    "875e-06,2.759375e-06,2.759375e-06,2.821875e-06,2.759375e-06,2.821875e-"
    "06,2.821875e-06,2.759375e-06,2.759375e-06,2.821875e-06,2.821875e-06,2."
    "821875e-06,2.821875e-06,2.821875e-06,2.759375e-06,2.759375e-06,2.75937"
    "5e-06,2.821875e-06,2.821875e-06,2.759375e-06,2.821875e-06,2.821875e-06"
    ",2.821875e-06,2.759375e-06,2.821875e-06,2.759375e-06,2.821875e-06]}"
};

// Statistics about target distance wrt. SOCS origin
const MetaDataItemInfo RDB_RIEGL_RANGE_STATISTICS = {
    /* name        */ "riegl.range_statistics",
    /* title       */ "Range Statistics",
    /* description */ "Statistics about target distance wrt. SOCS origin",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Range Statistics\",\"type\":\"object\",\"$schema\":\"http://json-sch"
    "ema.org/draft-04/schema#\",\"properties\":{\"maximum\":{\"description\":\"Maxi"
    "mum value\",\"type\":\"number\"},\"average\":{\"description\":\"Average "
    "value\",\"type\":\"number\"},\"std_dev\":{\"description\":\"Standard "
    "deviation\",\"type\":\"number\"},\"minimum\":{\"description\":\"Minimum "
    "value\",\"type\":\"number\"}},\"description\":\"Statistics about target "
    "distance wrt. SOCS "
    "origin\",\"required\":[\"minimum\",\"average\",\"maximum\",\"std_dev\"]}"
    ,
    /* example: */
    "{\"maximum\":574.35,\"average\":15.49738,\"std_dev\":24.349,\"minimum\":0.919}"
};

// Receiver Internals
const MetaDataItemInfo RDB_RIEGL_RECEIVER_INTERNALS = {
    /* name        */ "riegl.receiver_internals",
    /* title       */ "Receiver Internals",
    /* description */ "Receiver Internals",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Receiver Internals\",\"type\":\"object\",\"$schema\":\"http://json-s"
    "chema.org/draft-04/schema#\",\"properties\":{\"si\":{\"maximum\":255,\"descrip"
    "tion\":\"Start index (hw_start)\",\"type\":\"number\",\"minimum\":0},\"tbl\":{\"it"
    "ems\":{\"$ref\":\"#/definitions/fp_table\"},\"minItems\":1,\"description\":\"var"
    "ious internal data\",\"type\":\"array\"},\"a\":{\"maxItems\":256,\"items\":{\"type"
    "\":\"number\"},\"minItems\":1,\"description\":\"Amplitude "
    "[dB]\",\"type\":\"array\"},\"mw\":{\"maximum\":4095,\"description\":\"Maximum weig"
    "ht\",\"exclusiveMinimum\":true,\"type\":\"number\",\"minimum\":0},\"ex\":{\"descri"
    "ption\":\"DEPRECATED, use 'riegl.exponential_decomposition' "
    "instead\",\"type\":\"object\"},\"nt\":{\"maximum\":255,\"description\":\"Number of"
    " traces\",\"type\":\"integer\",\"minimum\":0},\"ns\":{\"maximum\":4095,\"descripti"
    "on\":\"Number of samples\",\"type\":\"integer\",\"minimum\":0},\"t\":{\"patternPro"
    "perties\":{\"^[0-9]+$\":{\"$ref\":\"#/definitions/fp\",\"description\":\"one "
    "field per channel, field name is channel index\"}},\"additionalPropertie"
    "s\":false,\"type\":\"object\"},\"sr\":{\"description\":\"Sample rate [Hz]\",\"excl"
    "usiveMinimum\":true,\"type\":\"number\",\"minimum\":0}},\"description\":\"Receiv"
    "er Internals\",\"definitions\":{\"fp\":{\"properties\":{\"w\":{\"maxItems\":256,\""
    "items\":{\"maxItems\":5,\"items\":{\"type\":\"number\"},\"minItems\":5,\"type\":\"ar"
    "ray\"},\"minItems\":1,\"type\":\"array\"},\"s\":{\"maxItems\":256,\"items\":{\"maxIt"
    "ems\":4096,\"items\":{\"type\":\"number\"},\"minItems\":1,\"type\":\"array\"},\"minI"
    "tems\":1,\"type\":\"array\"}},\"description\":\"Fingerprint values\",\"type\":\"ob"
    "ject\",\"required\":[\"s\",\"w\"]},\"fp_table\":{\"desription\":\"scanner internal"
    " data\",\"properties\":{\"ny\":{\"max\":2048,\"min\":1,\"description\":\"number of"
    " y entries\",\"type\":\"integer\"},\"ch\":{\"max\":255,\"min\":0,\"description\":\"c"
    "hannel number\",\"type\":\"integer\"},\"tv\":{\"maxItems\":2048,\"items\":{\"oneOf"
    "\":[{\"$ref\":\"#/definitions/fp_table_row\"},{\"type\":\"number\"}]},\"minItems"
    "\":1,\"type\":\"array\"},\"nx\":{\"max\":2048,\"min\":1,\"description\":\"number of "
    "x entries\",\"type\":\"integer\"},\"tc\":{\"max\":255,\"min\":0,\"description\":\"ta"
    "ble data type code\",\"type\":\"integer\"}},\"required\":[\"ch\",\"tc\",\"nx\",\"ny\""
    ",\"tv\"],\"type\":\"object\"},\"fp_table_row\":{\"maxItems\":2048,\"items\":{\"type"
    "\":\"number\"},\"minItems\":1,\"type\":\"array\"}}}"
    ,
    /* example: */
    "{\"si\":48,\"tbl\":[{\"ny\":2,\"ch\":0,\"tv\":[[1,2,3,4,5],[1.1,2.2,3.3,4.4,5.5]"
    "],\"nx\":5,\"tc\":1}],\"a\":[-1.55],\"mw\":31,\"nt\":128,\"ns\":400,\"t\":{\"0\":{\"w\":"
    "[[78,86,126,134,31],[78,86,126,134,31]],\"s\":[[1.23,4.56],[7.89,0.12]]}"
    ",\"1\":{\"w\":[[78,86,126,134,31],[78,86,126,134,31]],\"s\":[[1.23,4.56],[7."
    "89,0.12]]}},\"sr\":7959997000.0}"
};

// Lookup table for reflectance calculation based on amplitude and range
const MetaDataItemInfo RDB_RIEGL_REFLECTANCE_CALCULATION = {
    /* name        */ "riegl.reflectance_calculation",
    /* title       */ "Reflectance Calculation Table",
    /* description */ "Lookup table for reflectance calculation based on amplitude and range",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Reflectance Calculation Table\",\"type\":\"object\",\"$schema\":\"ht"
    "tp://json-schema.org/draft-04/schema#\",\"properties\":{\"delta\":{\"type\":\""
    "number\",\"description\":\"Delta between table entries [m], first entry is"
    " at range = 0 m\"},\"content\":{\"maxItems\":2000,\"items\":{\"type\":\"number\"}"
    ",\"minItems\":1,\"type\":\"array\",\"description\":\"Correction value [dB] to "
    "be added to the amplitude\"}},\"description\":\"Lookup table for "
    "reflectance calculation based on amplitude and "
    "range\",\"required\":[\"delta\",\"content\"]}"
    ,
    /* example: */
    "{\"delta\":0.150918,\"content\":[-33.01]}"
};

// Range-dependent and scan-angle-dependent correction of reflectance reading
const MetaDataItemInfo RDB_RIEGL_REFLECTANCE_CORRECTION = {
    /* name        */ "riegl.reflectance_correction",
    /* title       */ "Near-Range Reflectance Correction",
    /* description */ "Range-dependent and scan-angle-dependent correction of reflectance reading",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Near-range reflectance correction\",\"type\":\"object\",\"$schema\""
    ":\"http://json-schema.org/draft-04/schema#\",\"properties\":{\"reflectance_"
    "correction_db\":{\"items\":{\"items\":{\"type\":\"number\",\"description\":\"colum"
    "ns (each value corresponds to an "
    "angle)\"},\"type\":\"array\",\"description\":\"rows (each array corresponds to"
    " a range)\"},\"description\":\"Near range reflectance correction in dB as "
    "a function of range over angle\",\"type\":\"array\"},\"ranges_m\":{\"items\":{\""
    "type\":\"number\"},\"description\":\"Range [m]\",\"type\":\"array\"},\"line_angles"
    "_deg\":{\"items\":{\"type\":\"number\"},\"description\":\"Angle "
    "[deg]\",\"type\":\"array\"}},\"description\":\"Range-dependent and "
    "scan-angle-dependent correction of reflectance reading\",\"required\":[\"r"
    "anges_m\",\"line_angles_deg\",\"reflectance_correction_db\"]}"
    ,
    /* example: */
    "{\"reflectance_correction_db\":[[0.8,0.7,0.6,0.5,0.4,0.3,0.2,0.1,0.05,0."
    "01],[0.8,0.7,0.6,0.5,0.4,0.3,0.2,0.1,0.05,0.01],[0.8,0.7,0.6,0.5,0.4,0"
    ".3,0.2,0.1,0.05,0.01],[0.8,0.7,0.6,0.5,0.4,0.3,0.2,0.1,0.05,0.01]],\"ra"
    "nges_m\":[0.0,1.0,2.0,3.0],\"line_angles_deg\":[0.0,0.5,1.0,1.5,1.0,2.0,2"
    ".5,3.0,3.5,4.0]}"
};

// Scan pattern description
const MetaDataItemInfo RDB_RIEGL_SCAN_PATTERN = {
    /* name        */ "riegl.scan_pattern",
    /* title       */ "Scan Pattern",
    /* description */ "Scan pattern description",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"properties\":{\"li"
    "ne\":{\"properties\":{\"start\":{\"maximum\":360.0,\"type\":\"number\",\"descripti"
    "on\":\"Start angle in SOCS [deg]\",\"minimum\":0.0},\"increment\":{\"maximum\":"
    "90.0,\"type\":\"number\",\"description\":\"Increment of angle in SOCS [deg]\","
    "\"minimum\":0.0},\"stop\":{\"maximum\":720.0,\"type\":\"number\",\"description\":\""
    "Stop angle in SOCS [deg]\",\"minimum\":0.0},\"program\":{\"$ref\":\"#/definiti"
    "ons/program\"}},\"type\":\"object\",\"description\":\"Line Scan Pattern\",\"requ"
    "ired\":[\"start\",\"stop\",\"increment\"]},\"rectangular\":{\"properties\":{\"thet"
    "a_increment\":{\"maximum\":90.0,\"type\":\"number\",\"description\":\"Increment "
    "of theta angle in SOCS [deg]\",\"minimum\":0.0},\"theta_start\":{\"maximum\":"
    "180.0,\"type\":\"number\",\"description\":\"Start theta angle in SOCS [deg]\","
    "\"minimum\":0.0},\"phi_start\":{\"maximum\":360.0,\"type\":\"number\",\"descripti"
    "on\":\"Start phi angle in SOCS [deg]\",\"minimum\":0.0},\"phi_stop\":{\"maximu"
    "m\":720.0,\"type\":\"number\",\"description\":\"Stop phi angle in SOCS [deg]\","
    "\"minimum\":0.0},\"theta_stop\":{\"maximum\":180.0,\"type\":\"number\",\"descript"
    "ion\":\"Stop theta angle in SOCS [deg]\",\"minimum\":0.0},\"program\":{\"$ref\""
    ":\"#/definitions/program\"},\"phi_increment\":{\"maximum\":90.0,\"type\":\"numb"
    "er\",\"description\":\"Increment of phi angle in SOCS "
    "[deg]\",\"minimum\":0.0}},\"type\":\"object\",\"description\":\"Rectangular "
    "Field Of View Scan Pattern\",\"required\":[\"phi_start\",\"phi_stop\",\"phi_in"
    "crement\",\"theta_start\",\"theta_stop\",\"theta_increment\"]},\"segments\":{\"p"
    "roperties\":{\"list\":{\"items\":{\"properties\":{\"start\":{\"maximum\":360.0,\"t"
    "ype\":\"number\",\"description\":\"Start angle in SOCS [deg]\",\"minimum\":0.0}"
    ",\"increment\":{\"maximum\":90.0,\"type\":\"number\",\"description\":\"Increment "
    "of angle in SOCS [deg]\",\"minimum\":0.0},\"stop\":{\"maximum\":720.0,\"type\":"
    "\"number\",\"description\":\"Stop angle in SOCS "
    "[deg]\",\"minimum\":0.0}},\"type\":\"object\",\"description\":\"Line Scan Segmen"
    "t\",\"required\":[\"start\",\"stop\",\"increment\"]},\"type\":\"array\"},\"program\":"
    "{\"$ref\":\"#/definitions/program\"}},\"type\":\"object\",\"description\":\"Segme"
    "nted Line Scan Pattern\",\"required\":[\"list\"]}},\"title\":\"Scan "
    "Pattern\",\"description\":\"Scan pattern description\",\"definitions\":{\"prog"
    "ram\":{\"properties\":{\"name\":{\"type\":\"string\",\"description\":\"Name of "
    "measurement program\"},\"laser_prr\":{\"exclusiveMinimum\":false,\"type\":\"nu"
    "mber\",\"description\":\"Laser Pulse Repetition Rate "
    "[Hz]\",\"minimum\":0}},\"type\":\"object\",\"description\":\"Measurement "
    "program\",\"required\":[\"name\"]}}}"
    ,
    /* example: */
    "{\"rectangular\":{\"theta_increment\":0.04,\"theta_start\":30.0,\"phi_start\":"
    "45.0,\"phi_stop\":270.0,\"theta_stop\":130.0,\"program\":{\"name\":\"High "
    "Speed\",\"laser_prr\":100000.0},\"phi_increment\":0.04}}"
};

// Details about laser shot files
const MetaDataItemInfo RDB_RIEGL_SHOT_INFO = {
    /* name        */ "riegl.shot_info",
    /* title       */ "Shot Information",
    /* description */ "Details about laser shot files",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"properties\":{\"sh"
    "ot_file\":{\"properties\":{\"file_uuid\":{\"type\":\"string\",\"description\":\"Fi"
    "le's Universally Unique Identifier (RFC "
    "4122)\"},\"file_extension\":{\"type\":\"string\",\"description\":\"Shot file "
    "extension, without the leading "
    "dot\"}},\"required\":[\"file_extension\"],\"type\":\"object\"}},\"title\":\"Shot "
    "Information\",\"description\":\"Details about laser shot "
    "files\",\"type\":\"object\"}"
    ,
    /* example: */
    "{\"shot_file\":{\"file_uuid\":\"26a00815-67c0-4bff-8fe8-c577378fe663\",\"file"
    "_extension\":\"sodx\"}}"
};

// Point filters applied in addition to the application-defined filters
const MetaDataItemInfo RDB_RIEGL_STORED_FILTERS = {
    /* name        */ "riegl.stored_filters",
    /* title       */ "Stored Filters",
    /* description */ "Point filters applied in addition to the application-defined filters",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Stored filters\",\"type\":\"object\",\"$schema\":\"http://json-schem"
    "a.org/draft-04/schema#\",\"properties\":{\"filters\":{\"items\":{\"properties\""
    ":{\"filter\":{\"type\":\"string\",\"description\":\"The RDB filter string to "
    "apply (e.g. when reading points or index), details see documentation "
    "of function "
    "select()\"},\"activated\":{\"type\":\"boolean\",\"description\":\"Apply ('true')"
    " or ignore ('false') this "
    "filter\"},\"description\":{\"type\":\"string\",\"description\":\"A brief "
    "description of the filter (e.g. for display in a "
    "tooltip)\"},\"title\":{\"type\":\"string\",\"description\":\"A short filter "
    "title (e.g. for display in a selection "
    "list)\"}},\"type\":\"object\",\"description\":\"Point filter definition\",\"requ"
    "ired\":[\"activated\",\"title\",\"description\",\"filter\"]},\"type\":\"array\",\"de"
    "scription\":\"List of point "
    "filters\"},\"activated\":{\"type\":\"boolean\",\"description\":\"Apply ('true') "
    "or ignore ('false') all filters\"}},\"description\":\"Point filters "
    "applied in addition to the application-defined "
    "filters\",\"required\":[\"activated\",\"filters\"]}"
    ,
    /* example: */
    "{\"filters\":[{\"filter\":\"riegl.mta_uncertain_point == "
    "0\",\"activated\":false,\"description\":\"Ignore points with uncertain MTA "
    "zone assignment\",\"title\":\"Ignore uncertain points from MTA "
    "resolution\"},{\"filter\":\"riegl.dyntrig_uncertain_point == "
    "0\",\"activated\":false,\"description\":\"Ignore points with an echo signal "
    "amplitude below the dynamic trigger\",\"title\":\"Ignore points below "
    "dynamic trigger level\"},{\"filter\":\"riegl.point_outside_aabb_bocs == "
    "0\",\"activated\":false,\"description\":\"Ignore points that are outside the"
    " outer AABB in BOCS\",\"title\":\"Ignore points outside outer "
    "AABB\"},{\"filter\":\"riegl.point_inside_aabb_bocs == "
    "0\",\"activated\":false,\"description\":\"Ignore points that are inside the "
    "inner AABB in BOCS\",\"title\":\"Ignore points inside inner "
    "AABB\"}],\"activated\":true}"
};

// Details of major system components like lidar sensors, cameras and navigation devices
const MetaDataItemInfo RDB_RIEGL_SYSTEM_DESCRIPTION = {
    /* name        */ "riegl.system_description",
    /* title       */ "System Description",
    /* description */ "Details of major system components like lidar sensors, cameras and navigation devices",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"System Description\",\"type\":\"object\",\"$schema\":\"http://json-s"
    "chema.org/draft-04/schema#\",\"properties\":{\"author\":{\"type\":\"string\",\"d"
    "escription\":\"Name of the author that created the "
    "document\"},\"system\":{\"type\":\"object\",\"description\":\"The actual system "
    "description, details see documentation of RIEGL System "
    "Description\"},\"$class\":{\"type\":\"string\",\"description\":\"Object class na"
    "me\",\"enum\":[\"Document\"]},\"version\":{\"type\":\"string\",\"description\":\"Doc"
    "ument format "
    "version\"},\"timestamp\":{\"type\":\"string\",\"description\":\"Date and time of"
    " creation (e.g. 2019-06-19T13:32:10+02:00)\"}},\"description\":\"Details "
    "of major system components like lidar sensors, cameras and navigation "
    "devices\",\"required\":[\"$class\",\"version\",\"author\",\"timestamp\",\"system\"]"
    "}"
    ,
    /* example: */
    "{\"author\":\"RIEGL LMS GmbH, Austria\",\"system\":{\"details\":\"see "
    "documentation of RIEGL System Description\"},\"$class\":\"Document\",\"versi"
    "on\":\"1.2.1\",\"timestamp\":\"2022-09-30T11:56:26+00:00\"}"
};

// Conversion of background radiation raw values to temperatures in °C
const MetaDataItemInfo RDB_RIEGL_TEMPERATURE_CALCULATION = {
    /* name        */ "riegl.temperature_calculation",
    /* title       */ "Temperature Calculation Table",
    /* description */ "Conversion of background radiation raw values to temperatures in °C",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Temperature Calculation Table\",\"type\":\"object\",\"$schema\":\"ht"
    "tp://json-schema.org/draft-04/schema#\",\"properties\":{\"Si\":{\"$ref\":\"#/d"
    "efinitions/conversion_table\",\"description\":\"Conversion table for Si ch"
    "annel\"},\"InGaAs_Si_Difference\":{\"$ref\":\"#/definitions/conversion_table"
    "\",\"description\":\"Conversion table for InGaAs - Si difference\"},\"InGaAs"
    "\":{\"$ref\":\"#/definitions/conversion_table\",\"description\":\"Conversion "
    "table for InGaAs channel\"}},\"description\":\"Conversion of background "
    "radiation raw values to temperatures in \\u00b0C\",\"definitions\":{\"conve"
    "rsion_table\":{\"properties\":{\"value\":{\"items\":{\"type\":\"number\"},\"descri"
    "ption\":\"LSB [1]\",\"type\":\"array\"},\"temperature\":{\"items\":{\"type\":\"numbe"
    "r\"},\"description\":\"Temperature [\\u00b0C]\",\"type\":\"array\"}},\"required\":"
    "[\"value\",\"temperature\"],\"type\":\"object\"}}}"
    ,
    /* example: */
    "{\"Si\":{\"value\":[0.0,64.00097659230323,128.0019531846065,192.0029297769"
    "097,256.0039063692129,320.00488296151616,384.0058595538194,448.0068361"
    "461226,512.0078127384259],\"temperature\":[546.300048828125,548.81640512"
    "12026,551.3143938500972,554.0144257850053,556.604252334815,559.2124464"
    "488079,561.8022729986177,564.4104671126105,567.0002936624203]},\"InGaAs"
    "_Si_Difference\":{\"value\":[1000.0,1100.090029602954,1200.04425183874,13"
    "00.1342814416948,1400.0885036774805,1500.0427259132668,1600.1327555162"
    "209,1700.0869777520065,1800.0411999877924],\"temperature\":[1749.9771111"
    "17893,1749.977111117893,1749.977111117893,1749.977111117893,1749.97711"
    "1117893,1749.977111117893,1744.7813348796044,1681.9971312601092,1622.3"
    "944822534868]},\"InGaAs\":{\"value\":[0.0,64.00097659230323,128.0019531846"
    "065,192.0029297769097,256.0039063692129,320.00488296151616,384.0058595"
    "538194,448.0068361461226,512.0078127384259],\"temperature\":[307.2219672"
    "2535614,309.1153478247277,311.1188086915047,313.10025350480055,315.213"
    "7946389828,317.2172555057597,319.2207163725366,321.2021611858325,323.3"
    "157023200148]}}"
};

// Base of timestamps (epoch)
const MetaDataItemInfo RDB_RIEGL_TIME_BASE = {
    /* name        */ "riegl.time_base",
    /* title       */ "Time Base",
    /* description */ "Base of timestamps (epoch)",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Time Base\",\"type\":\"object\",\"$schema\":\"http://json-schema.org"
    "/draft-04/schema#\",\"properties\":{\"system\":{\"type\":\"string\",\"descriptio"
    "n\":\"Time system (time standard)\",\"enum\":[\"unknown\",\"UTC\",\"GPS\"]},\"sour"
    "ce\":{\"type\":\"string\",\"description\":\"Timestamp source\",\"enum\":[\"unknown"
    "\",\"RTC\",\"GNSS\"]},\"epoch\":{\"type\":\"string\",\"description\":\"Date and time"
    " of timestamp '0' as proposed by RFC 3339 (e.g. "
    "2015-10-27T00:00:00+01:00).\"}},\"description\":\"Base of timestamps "
    "(epoch)\",\"required\":[\"epoch\",\"source\"]}"
    ,
    /* example: */
    "{\"system\":\"UTC\",\"source\":\"GNSS\",\"epoch\":\"2015-10-27T00:00:00+00:00\"}"
};

// Details about position+orientation files
const MetaDataItemInfo RDB_RIEGL_TRAJECTORY_INFO = {
    /* name        */ "riegl.trajectory_info",
    /* title       */ "Trajectory Information",
    /* description */ "Details about position+orientation files",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Trajectory Information\",\"type\":\"object\",\"$schema\":\"http://js"
    "on-schema.org/draft-04/schema#\",\"properties\":{\"time_interval\":{\"proper"
    "ties\":{\"maximum\":{\"type\":\"number\",\"description\":\"Maximum time interval"
    " [s]\"},\"average\":{\"type\":\"number\",\"description\":\"Average time interval"
    " [s]\"},\"std_dev\":{\"type\":\"number\",\"description\":\"Standard deviation of"
    " intervals [s]\"},\"minimum\":{\"type\":\"number\",\"description\":\"Minimum "
    "time interval [s]\"}},\"type\":\"object\",\"description\":\"Time interval stat"
    "istics\",\"required\":[\"minimum\",\"average\",\"maximum\",\"std_dev\"]},\"project"
    "\":{\"type\":\"string\",\"description\":\"Project "
    "name\"},\"location\":{\"type\":\"string\",\"description\":\"Project location, "
    "e.g. "
    "city/state/country\"},\"company\":{\"type\":\"string\",\"description\":\"Company"
    " name\"},\"field_of_application\":{\"type\":\"string\",\"description\":\"Field "
    "of application\",\"enum\":[\"unknown\",\"SLS\",\"TLS\",\"KLS\",\"MLS\",\"ULS\",\"ALS\","
    "\"BLS\",\"ILS\"]},\"device\":{\"type\":\"string\",\"description\":\"Navigation "
    "device, e.g. "
    "name/type/serial\"},\"settings\":{\"type\":\"string\",\"description\":\"Settings"
    " used to calculate the trajectory (descriptive "
    "text)\"},\"navigation_frame\":{\"type\":\"string\",\"description\":\"Navigation "
    "frame (NED: North-East-Down, ENU: East-North-Up)\",\"enum\":[\"unknown\",\"N"
    "ED\",\"ENU\"]},\"software\":{\"type\":\"string\",\"description\":\"Software that "
    "calculated the trajectory (this may be the same or different software "
    "than the one that created the file)\"}},\"description\":\"Details about "
    "position+orientation "
    "files\",\"required\":[\"time_interval\",\"navigation_frame\"]}"
    ,
    /* example: */
    "{\"time_interval\":{\"maximum\":0.005004883,\"average\":0.00500053,\"std_dev\""
    ":5.51e-07,\"minimum\":0.00500032},\"project\":\"Campaign "
    "3\",\"location\":\"Horn\",\"company\":\"RIEGL "
    "LMS\",\"field_of_application\":\"MLS\",\"device\":\"IMU Model 12/1, Serial# 12"
    "345\",\"settings\":\"default\",\"navigation_frame\":\"NED\",\"software\":\"Navigat"
    "ion Software XYZ\"}"
};

// Trigger-Timestamping-Unit/IP configuration
const MetaDataItemInfo RDB_RIEGL_TTIP_CONFIGURATION = {
    /* name        */ "riegl.ttip_configuration",
    /* title       */ "TTIP Configuration",
    /* description */ "Trigger-Timestamping-Unit/IP configuration",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"TTIP Configuration\",\"type\":\"object\",\"$schema\":\"http://json-s"
    "chema.org/draft-04/schema#\",\"properties\":{\"ext_bitmask\":{\"type\":\"integ"
    "er\",\"description\":\"defines which of the internal pulse generators are "
    "to be "
    "started\",\"minimum\":0},\"ext_signal\":{\"type\":\"integer\",\"description\":\"0 "
    ".. use input signal, 1 .. use output signal of channel\",\"minimum\":0},\""
    "input_descr\":{\"type\":\"string\",\"description\":\"descriptive "
    "string\"},\"in_min_duration\":{\"type\":\"integer\",\"description\":\"input "
    "signals with smaller pulse durations are ignored [0.1 "
    "msec]\",\"minimum\":0},\"input_usage\":{\"type\":\"integer\",\"description\":\"0 "
    ".. deactivated, 1 .. just detecting and timestamping\",\"minimum\":0},\"in"
    "_max_delay\":{\"type\":\"integer\",\"description\":\"maximum delay to output "
    "pulse before fake event is generated [0.1 msec], zero indicates that "
    "no fake events are generated\",\"minimum\":0},\"in_polarity\":{\"type\":\"inte"
    "ger\",\"description\":\"0 .. positive edge, 1 .. negative "
    "edge\",\"minimum\":0},\"dmi_incr\":{\"type\":\"integer\",\"description\":\"dmi, "
    "increment in "
    "ticks\"},\"out_duration\":{\"type\":\"integer\",\"description\":\"output pulse "
    "duration  [0.1 msec]\",\"minimum\":0},\"out_interval\":{\"type\":\"integer\",\"d"
    "escription\":\"output pulse interval  [0.1 msec]\",\"minimum\":0},\"dmi_min_"
    "time\":{\"type\":\"integer\",\"description\":\"dmi, minimum time interval "
    "between trigger events [0.1 msec]\",\"minimum\":0},\"out_delay\":{\"type\":\"i"
    "nteger\",\"description\":\"output pulse initial delay after start  [0.1 ms"
    "ec]\",\"minimum\":0},\"ext_delay\":{\"type\":\"integer\",\"description\":\"externa"
    "l trigger, time delay [0.1 "
    "msec]\",\"minimum\":0},\"output_usage\":{\"type\":\"integer\",\"description\":\"0 "
    ".. no output, 1 .. output derived from internal clock, 2 .. output "
    "derived from dmi, 3 .. output derived from external signal, 4 .. "
    "output static low, 5 .. output static high\",\"minimum\":0},\"ext_subdivid"
    "er\":{\"type\":\"integer\",\"description\":\"reduces the frequency, default 1\""
    ",\"minimum\":0},\"dmi_dist_per_tick\":{\"type\":\"number\",\"description\":\"dmi,"
    " distance per tick, just informative "
    "[m]\"},\"output_descr\":{\"type\":\"string\",\"description\":\"descriptive "
    "string\"},\"dmi_max_time\":{\"type\":\"integer\",\"description\":\"dmi, maximum "
    "time interval between trigger events [0.1 "
    "msec]\",\"minimum\":0},\"out_polarity\":{\"type\":\"integer\",\"description\":\"0 "
    ".. positive edge, 1 .. negative edge\",\"minimum\":0},\"out_num_pulses\":{\""
    "type\":\"integer\",\"description\":\"number of output pulses to be "
    "generated, 0 .. infinite\",\"minimum\":0},\"in_max_duration\":{\"type\":\"inte"
    "ger\",\"description\":\"stops measurement of pulse duration of input "
    "signal [0.1 msec]\",\"minimum\":0},\"ttip_version\":{\"type\":\"integer\",\"desc"
    "ription\":\"following main.sub.ss.sss\",\"minimum\":0},\"num_channel\":{\"type"
    "\":\"integer\",\"description\":\"number of input/output "
    "channels\",\"minimum\":0},\"channel\":{\"type\":\"integer\",\"description\":\"ID "
    "of input/output channel\",\"minimum\":0},\"ext_channel\":{\"type\":\"integer\","
    "\"description\":\"ID of channel used as external trigger input, 32 "
    "indicates "
    "none\",\"minimum\":0}},\"description\":\"Trigger-Timestamping-Unit/IP config"
    "uration\",\"required\":[\"ttip_version\",\"num_channel\",\"ext_channel\",\"ext_s"
    "ignal\",\"ext_delay\",\"ext_subdivider\",\"ext_bitmask\",\"dmi_incr\",\"dmi_min_"
    "time\",\"dmi_max_time\",\"dmi_dist_per_tick\",\"channel\",\"output_descr\",\"inp"
    "ut_descr\",\"output_usage\",\"input_usage\",\"out_polarity\",\"out_duration\",\""
    "out_interval\",\"out_delay\",\"out_num_pulses\",\"in_polarity\",\"in_min_durat"
    "ion\",\"in_max_duration\",\"in_max_delay\"]}"
    ,
    /* example: */
    "{\"ext_bitmask\":0,\"ext_signal\":0,\"input_descr\":\"Port 1 - Exposure\",\"in_"
    "min_duration\":0,\"input_usage\":1,\"in_max_delay\":9990,\"in_polarity\":0,\"d"
    "mi_incr\":0,\"out_duration\":300,\"out_interval\":10000,\"dmi_min_time\":0,\"o"
    "ut_delay\":0,\"ext_delay\":0,\"output_usage\":1,\"ext_subdivider\":1,\"dmi_dis"
    "t_per_tick\":0.0,\"output_descr\":\"Port 1 - Trigger\",\"dmi_max_time\":0,\"ou"
    "t_polarity\":0,\"out_num_pulses\":1,\"in_max_duration\":10000,\"ttip_version"
    "\":1,\"num_channel\":9,\"channel\":0,\"ext_channel\":32}"
};

// Details about vertex file
const MetaDataItemInfo RDB_RIEGL_VERTEX_INFO = {
    /* name        */ "riegl.vertex_info",
    /* title       */ "Vertex Information",
    /* description */ "Details about vertex file",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"properties\":{\"ve"
    "rtex_file\":{\"properties\":{\"file_uuid\":{\"type\":\"string\",\"description\":\""
    "File's Universally Unique Identifier (RFC "
    "4122)\"},\"file_extension\":{\"type\":\"string\",\"description\":\"Vertex file "
    "extension, without the leading "
    "dot\"}},\"required\":[\"file_extension\"],\"type\":\"object\"}},\"title\":\"Vertex"
    " Information\",\"description\":\"Details about vertex "
    "file\",\"type\":\"object\"}"
    ,
    /* example: */
    "{\"vertex_file\":{\"file_uuid\":\"51534d95-d71f-4f36-ae1a-3e63a21fd1c7\",\"fi"
    "le_extension\":\"vtx\"}}"
};

// Details about the voxels contained in the file
const MetaDataItemInfo RDB_RIEGL_VOXEL_INFO = {
    /* name        */ "riegl.voxel_info",
    /* title       */ "Voxel Information",
    /* description */ "Details about the voxels contained in the file",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Voxel Information\",\"type\":\"object\",\"$schema\":\"http://json-sc"
    "hema.org/draft-04/schema#\",\"oneOf\":[{\"properties\":{\"voxel_type\":{\"$ref"
    "\":\"#/definitions/voxel_type\"},\"size\":{\"oneOf\":[{\"$ref\":\"#/definitions/"
    "voxel_size\"},{\"$ref\":\"#/definitions/voxel_size_cubic\"}],\"description\":"
    "\"Size of voxels in file coordinate system.\"},\"shape_thresholds\":{\"$ref"
    "\":\"#/definitions/shape_thresholds\"},\"voxel_origin\":{\"$ref\":\"#/definiti"
    "ons/voxel_origin_enum\"}},\"additionalProperties\":false,\"required\":[\"siz"
    "e\",\"voxel_origin\",\"voxel_type\"]},{\"properties\":{\"reference_point\":{\"ma"
    "xItems\":2,\"items\":{\"maximum\":180,\"type\":\"number\",\"minimum\":-180},\"minI"
    "tems\":2,\"type\":\"array\",\"description\":\"Point in WGS84 geodetic decimal "
    "degree (EPSG:4326) that was used to compute the projection distortion "
    "parameters. The coefficient order is latitude, longitude. Only voxels "
    "with corresponding geo_tag, voxel_size and reference_point can be "
    "reliably processed together. This entry is available for voxel files "
    "in projected CRS only.\"},\"voxel_type\":{\"$ref\":\"#/definitions/voxel_typ"
    "e\"},\"size\":{\"$ref\":\"#/definitions/voxel_size\",\"description\":\"Size of "
    "voxels in file coordinate system.\"},\"shape_thresholds\":{\"$ref\":\"#/defi"
    "nitions/shape_thresholds\"},\"voxel_origin\":{\"oneOf\":[{\"$ref\":\"#/definit"
    "ions/voxel_origin_enum\"},{\"$ref\":\"#/definitions/voxel_origin_point\",\"d"
    "escription\":\"The base point of the voxel grid. Used together with "
    "<tt>voxel_size</tt> and <tt>voxel_index</tt> to compute actual point c"
    "oordinates.\"}]},\"size_llcs\":{\"$ref\":\"#/definitions/voxel_size\",\"descri"
    "ption\":\"Size of voxels in a locally levelled cartesian coordinate "
    "system (xyz). This is only used for voxels based on a map projection.\""
    "}},\"additionalProperties\":false,\"required\":[\"reference_point\",\"size_ll"
    "cs\",\"size\",\"voxel_origin\",\"voxel_type\"]}],\"description\":\"Details about"
    " the voxels contained in the file\",\"definitions\":{\"voxel_origin_enum\":"
    "{\"default\":\"corner\",\"enum\":[\"center\",\"corner\"],\"description\":\"Defines "
    "whether the voxel's center or a corner is placed on CRS origin <tt>(0/"
    "0/0)</tt>.\"},\"voxel_type\":{\"default\":\"centroid\",\"enum\":[\"center\",\"cent"
    "roid\",\"index\"],\"description\":\"Whether a point in a voxel represents "
    "its center or its centroid. If type is <tt>index</tt> there is no "
    "point but only an integer voxel index.\"},\"voxel_origin_point\":{\"maxIte"
    "ms\":3,\"items\":{\"type\":\"number\"},\"minItems\":3,\"type\":\"array\",\"descripti"
    "on\":\"Origin point for all voxel indices in voxel CRS.\"},\"shape_thresho"
    "lds\":{\"properties\":{\"line\":{\"minimum\":1,\"exclusiveMinimum\":true,\"descr"
    "iption\":\"If the biggest eigenvalue is bigger than the median "
    "eigenvalue * line_threshold, the voxel is considered a line.\",\"type\":\""
    "number\"},\"plane\":{\"type\":\"number\",\"minimum\":0,\"maximum\":1,\"exclusiveMa"
    "ximum\":true,\"exclusiveMinimum\":true,\"description\":\"If the smallest "
    "eigenvalue is smaller than the median eigenvalue * plane_threshold, "
    "the voxel is considered a "
    "plane.\"}},\"required\":[\"plane\",\"line\"],\"description\":\"Thresholds used "
    "to compute the voxel's shape_id value.\",\"type\":\"object\"},\"edge_length\""
    ":{\"minimum\":0,\"exclusiveMinimum\":true,\"description\":\"Length of voxel "
    "edge [m].\",\"type\":\"number\"},\"voxel_size_cubic\":{\"$ref\":\"#/definitions/"
    "edge_length\",\"type\":\"number\"},\"voxel_size\":{\"maxItems\":3,\"items\":{\"$re"
    "f\":\"#/definitions/edge_length\"},\"minItems\":3,\"type\":\"array\",\"descripti"
    "on\":\"Size of voxels.\"}}}"
    ,
    /* example: */
    "{\"reference_point\":[48,16],\"voxel_type\":\"centroid\",\"size\":[0.597164283"
    "4779395,0.5971642834779395,0.5143705304787237],\"shape_thresholds\":{\"li"
    "ne\":6,\"plane\":0.16},\"voxel_origin\":\"corner\",\"size_llcs\":[0.51565752528"
    "91171,0.5130835356683303,0.5143705304787237]}"
};

// Settings for waveform averaging
const MetaDataItemInfo RDB_RIEGL_WAVEFORM_AVERAGING_SETTINGS = {
    /* name        */ "riegl.waveform_averaging_settings",
    /* title       */ "Waveform Averaging Settings",
    /* description */ "Settings for waveform averaging",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Waveform Averaging Settings\",\"type\":\"object\",\"$schema\":\"http"
    "://json-schema.org/draft-04/schema#\",\"properties\":{\"trim\":{\"maximum\":0"
    ".5,\"default\":0,\"description\":\"Percentage for robust averaging.\",\"type\""
    ":\"number\",\"minimum\":0},\"num_shots\":{\"description\":\"Number of "
    "consecutive shots to be used for averaging.\",\"type\":\"integer\",\"minimum"
    "\":1},\"mta_zone\":{\"description\":\"Fixed MTA zone for "
    "averaging.\",\"type\":\"integer\",\"minimum\":1}},\"description\":\"Settings for"
    " waveform averaging\",\"required\":[\"num_shots\",\"mta_zone\"]}"
    ,
    /* example: */
    "{\"trim\":0.05,\"num_shots\":7,\"mta_zone\":1}"
};

// Details about waveform files
const MetaDataItemInfo RDB_RIEGL_WAVEFORM_INFO = {
    /* name        */ "riegl.waveform_info",
    /* title       */ "Waveform Information",
    /* description */ "Details about waveform files",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Waveform Information\",\"type\":\"object\",\"$schema\":\"http://json"
    "-schema.org/draft-04/schema#\",\"properties\":{\"range_offset_m\":{\"type\":\""
    "number\",\"description\":\"Calibrated device specific range offset for "
    "waveform analysis by system response fitting in meters.\"},\"sample_bloc"
    "k_file\":{\"properties\":{\"file_uuid\":{\"type\":\"string\",\"description\":\"Fil"
    "e's Universally Unique Identifier (RFC "
    "4122)\"},\"file_extension\":{\"type\":\"string\",\"description\":\"Sample block "
    "file extension, without the leading dot\"}},\"required\":[\"file_extension"
    "\"],\"type\":\"object\"},\"range_offset_waveform_samples_m\":{\"type\":\"number\""
    ",\"description\":\"Calibrated device specific range offset for waveform "
    "samples in meters.\"},\"sample_data_files\":{\"items\":{\"properties\":{\"chan"
    "nel_name\":{\"type\":\"string\",\"description\":\"Sample block channel name\"},"
    "\"laser_wavelength\":{\"minimum\":0,\"type\":\"number\",\"description\":\"Laser "
    "wavelength in meters (0 = unknown)\",\"exclusiveMinimum\":false},\"channel"
    "\":{\"type\":\"integer\",\"maximum\":255,\"minimum\":0,\"exclusiveMaximum\":false"
    ",\"exclusiveMinimum\":false,\"description\":\"Sample block channel number "
    "(255 = invalid)\"},\"sample_interval\":{\"minimum\":0,\"type\":\"number\",\"desc"
    "ription\":\"Sampling interval in seconds\",\"exclusiveMinimum\":false},\"fil"
    "e_uuid\":{\"type\":\"string\",\"description\":\"File's Universally Unique "
    "Identifier (RFC 4122)\"},\"delta_st\":{\"type\":\"number\",\"description\":\"res"
    "erved\"},\"file_extension\":{\"type\":\"string\",\"description\":\"Sample data "
    "file extension, without the leading dot\"},\"sample_bits\":{\"type\":\"integ"
    "er\",\"maximum\":32,\"minimum\":0,\"exclusiveMaximum\":false,\"exclusiveMinimu"
    "m\":false,\"description\":\"Bitwidth of samples (e.g. 10 bit, 12 bit)\"}},\""
    "required\":[\"channel\",\"channel_name\",\"sample_interval\",\"sample_bits\",\"l"
    "aser_wavelength\",\"delta_st\",\"file_extension\"],\"type\":\"object\"},\"type\":"
    "\"array\"}},\"description\":\"Details about waveform "
    "files\",\"required\":[\"sample_block_file\",\"sample_data_files\"]}"
    ,
    /* example: */
    "{\"range_offset_m\":3.1415,\"sample_block_file\":{\"file_uuid\":\"93a03615-66"
    "c0-4bea-8ff8-c577378fe660\",\"file_extension\":\"sbx\"},\"sample_data_files\""
    ":[{\"channel_name\":\"high_power\",\"laser_wavelength\":0,\"channel\":0,\"sampl"
    "e_interval\":1.00503e-09,\"file_uuid\":\"da084413-e3e8-4655-a122-071de8490"
    "d8e\",\"delta_st\":0,\"file_extension\":\"sp0\",\"sample_bits\":12},{\"channel_n"
    "ame\":\"low_power\",\"laser_wavelength\":0,\"channel\":1,\"sample_interval\":1."
    "00503e-09,\"file_uuid\":\"93585b5e-5ea9-43a1-947b-e7ba3be642d2\",\"delta_st"
    "\":0,\"file_extension\":\"sp1\",\"sample_bits\":12},{\"channel_name\":\"wwf\",\"la"
    "ser_wavelength\":0,\"channel\":5,\"sample_interval\":1.00503e-09,\"file_uuid"
    "\":\"9d2298c4-1036-464f-b5cb-1cf8e517f3a0\",\"delta_st\":0,\"file_extension\""
    ":\"sp5\",\"sample_bits\":12}],\"range_offset_waveform_samples_m \":7.283}"
};

// Scanner settings for waveform output
const MetaDataItemInfo RDB_RIEGL_WAVEFORM_SETTINGS = {
    /* name        */ "riegl.waveform_settings",
    /* title       */ "Waveform Settings",
    /* description */ "Scanner settings for waveform output",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"$schema\":\"http://json-schema.org/draft-04/schema#\",\"items\":{\"propert"
    "ies\":{\"pass_ampl_less\":{\"type\":\"number\",\"description\":\"Threshold for "
    "amplitude less than "
    "[dB]\"},\"pass_rng_less\":{\"type\":\"number\",\"description\":\"Threshold for "
    "range less than "
    "[m]\"},\"logic_expression\":{\"type\":\"string\",\"description\":\"Logic "
    "expression of smart waveforms "
    "filter\"},\"channel_idx_mask\":{\"type\":\"integer\",\"description\":\"Bit mask "
    "for channels which belong to sbl_name: Channel 0 = Bit0, Channel 1 = "
    "Bit1, "
    "...\"},\"pass_ampl_greater\":{\"type\":\"number\",\"description\":\"Threshold "
    "for amplitude greater than "
    "[dB]\"},\"sbl_name\":{\"type\":\"string\",\"description\":\"Name of sample "
    "block, e.g.: wfm, "
    "wwf\"},\"pass_dev_greater\":{\"type\":\"integer\",\"description\":\"Threshold "
    "for deviation greater than "
    "[1]\"},\"smart_enabled\":{\"type\":\"boolean\",\"description\":\"Smart waveform "
    "output enabled\"},\"enabled\":{\"type\":\"boolean\",\"description\":\"Waveform "
    "output "
    "enabled\"},\"pass_dev_less\":{\"type\":\"integer\",\"description\":\"Threshold "
    "for deviation less than "
    "[1]\"},\"pass_rng_greater\":{\"type\":\"number\",\"description\":\"Threshold for"
    " range greater than [m]\"}},\"required\":[\"sbl_name\",\"enabled\",\"channel_i"
    "dx_mask\"],\"type\":\"object\"},\"title\":\"Waveform "
    "Settings\",\"description\":\"Scanner settings for waveform "
    "output\",\"type\":\"array\"}"
    ,
    /* example: */
    "[{\"pass_ampl_less\":5.0,\"pass_rng_less\":13.11,\"channel_idx_mask\":11,\"pa"
    "ss_ampl_greater\":1.0,\"sbl_name\":\"wfm\",\"smart_enabled\":true,\"enabled\":t"
    "rue,\"pass_rng_greater\":9.27},{\"channel_idx_mask\":32,\"enabled\":false,\"s"
    "bl_name\":\"wwf\"}]"
};

// Window analysis data estimated from scandata and resulting filter parameters
const MetaDataItemInfo RDB_RIEGL_WINDOW_ANALYSIS = {
    /* name        */ "riegl.window_analysis",
    /* title       */ "Window Analysis",
    /* description */ "Window analysis data estimated from scandata and resulting filter parameters",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Window Analysis\",\"type\":\"object\",\"$schema\":\"http://json-sche"
    "ma.org/draft-04/schema#\",\"properties\":{\"settings\":{\"properties\":{\"ampl"
    "itude\":{\"properties\":{\"additive_value\":{\"type\":\"number\"},\"sigma_factor"
    "\":{\"type\":\"number\"}},\"required\":[\"sigma_factor\",\"additive_value\"],\"typ"
    "e\":\"object\"},\"range\":{\"properties\":{\"additive_value\":{\"type\":\"number\"}"
    ",\"sigma_factor\":{\"type\":\"number\"}},\"required\":[\"sigma_factor\",\"additiv"
    "e_value\"],\"type\":\"object\"}},\"required\":[\"range\",\"amplitude\"],\"type\":\"o"
    "bject\"},\"result\":{\"properties\":{\"range_mean\":{\"items\":{\"type\":\"number\""
    "},\"description\":\"[m]\",\"type\":\"array\"},\"amplitude_mean\":{\"items\":{\"type"
    "\":\"number\"},\"description\":\"[dB]\",\"type\":\"array\"},\"timestamp\":{\"items\":"
    "{\"type\":\"number\"},\"description\":\"[s]\",\"type\":\"array\"},\"amplitude_sigma"
    "\":{\"items\":{\"type\":\"number\"},\"description\":\"[dB]\",\"type\":\"array\"},\"ang"
    "le\":{\"items\":{\"type\":\"number\"},\"description\":\"[deg]\",\"type\":\"array\"},\""
    "range_sigma\":{\"items\":{\"type\":\"number\"},\"description\":\"[m]\",\"type\":\"ar"
    "ray\"},\"amplitude_offset\":{\"items\":{\"type\":\"number\"},\"description\":\"[dB"
    "]\",\"type\":\"array\"}},\"required\":[\"angle\",\"range_mean\",\"range_sigma\",\"am"
    "plitude_mean\",\"amplitude_sigma\",\"amplitude_offset\"],\"type\":\"object\"},\""
    "filter\":{\"properties\":{\"range_max\":{\"items\":{\"type\":\"number\"},\"descrip"
    "tion\":\"[m]\",\"type\":\"array\"},\"range_min\":{\"items\":{\"type\":\"number\"},\"de"
    "scription\":\"[m]\",\"type\":\"array\"},\"angle\":{\"items\":{\"type\":\"number\"},\"d"
    "escription\":\"[deg]\",\"type\":\"array\"},\"amplitude_max\":{\"items\":{\"type\":\""
    "number\"},\"description\":\"[dB]\",\"type\":\"array\"}},\"required\":[\"angle\",\"ra"
    "nge_min\",\"range_max\",\"amplitude_max\"],\"type\":\"object\"}},\"description\":"
    "\"Window analysis data estimated from scandata and resulting filter "
    "parameters\",\"required\":[\"result\",\"filter\",\"settings\"]}"
    ,
    /* example: */
    "{\"settings\":{\"amplitude\":{\"additive_value\":1.0,\"sigma_factor\":4},\"rang"
    "e\":{\"additive_value\":0.1,\"sigma_factor\":8}},\"result\":{\"range_mean\":[0."
    "1105621,0.1079564,0.1087088,0.1067261,0.1054582,0.1090412,0.102871,0.1"
    "019044,0.1051523,0.1058445,0.1031261],\"amplitude_mean\":[5.347396,5.263"
    "155,5.224655,5.179926,5.097782,5.116479,5.051756,4.983473,5.007885,5.0"
    "02441,4.982],\"timestamp\":[408.4441,411.4443],\"amplitude_sigma\":[0.4272"
    "844,0.4298479,0.4236816,0.4283583,0.4362353,0.4315141,0.4373984,0.4472"
    "798,0.4346001,0.4345487,0.4540681],\"angle\":[14.9,15.0,15.1,15.2,15.3,1"
    "5.4,15.5,15.6,15.7,15.8,15.9],\"range_sigma\":[0.01869652,0.02151435,0.0"
    "1747969,0.01918765,0.01945776,0.01934862,0.01955329,0.02225589,0.02229"
    "977,0.01899122,0.02009433],\"amplitude_offset\":[1.9,1.9]},\"filter\":{\"ra"
    "nge_max\":[0.424,0.425,0.426,0.427,0.428,0.428,0.429,0.43,0.431,0.431,0"
    ".432],\"range_min\":[-0.208,-0.21,-0.212,-0.214,-0.216,-0.218,-0.219,-0."
    "221,-0.223,-0.225,-0.227],\"angle\":[14.9,15.0,15.1,15.2,15.3,15.4,15.5,"
    "15.6,15.7,15.8,15.9],\"amplitude_max\":[8.04,8.01,7.99,7.96,7.93,7.9,7.8"
    "8,7.85,7.83,7.8,7.78]}}"
};

// Correction parameters for window glass echoes
const MetaDataItemInfo RDB_RIEGL_WINDOW_ECHO_CORRECTION = {
    /* name        */ "riegl.window_echo_correction",
    /* title       */ "Window Echo Correction",
    /* description */ "Correction parameters for window glass echoes",
    /* status      */ RDB_STATUS_OPTIONAL,

    /* schema: */
    "{\"title\":\"Window Echo Correction\",\"type\":\"object\",\"$schema\":\"http://js"
    "on-schema.org/draft-04/schema#\",\"properties\":{\"amplitude\":{\"properties"
    "\":{\"entries\":{\"description\":\"Number of amplitude entries\",\"type\":\"inte"
    "ger\",\"minimum\":1},\"maximum\":{\"description\":\"Maximum amplitude in "
    "dB\",\"type\":\"number\",\"minimum\":0.0},\"minimum\":{\"description\":\"Minimum "
    "amplitude in "
    "dB\",\"type\":\"number\",\"minimum\":0.0}},\"description\":\"Amplitude axis of "
    "correction table\",\"type\":\"object\",\"required\":[\"minimum\",\"maximum\",\"ent"
    "ries\"]},\"slices\":{\"items\":{\"properties\":{\"amplitude\":{\"description\":\"W"
    "indow echo amplitude of slice in dB\",\"type\":\"number\"},\"table\":{\"items\""
    ":{\"items\":{\"maxItems\":3,\"items\":{\"type\":\"number\",\"description\":\"Table "
    "cell (item 0: amplitude in dB, 1: range in m, 2: "
    "flags)\"},\"minItems\":3,\"description\":\"Table column (= range "
    "axis)\",\"type\":\"array\"},\"minItems\":1,\"description\":\"Table row (= "
    "amplitude "
    "axis)\",\"type\":\"array\"},\"minItems\":1,\"description\":\"Correction table "
    "(dimension defined by the 'amplitude' and 'range' "
    "objects)\",\"type\":\"array\"}},\"description\":\"Window echo correction "
    "parameter slice\",\"type\":\"object\",\"required\":[\"amplitude\",\"table\"]},\"ty"
    "pe\":\"array\"},\"range\":{\"properties\":{\"entries\":{\"description\":\"Number "
    "of range entries\",\"type\":\"integer\",\"minimum\":1},\"maximum\":{\"maximum\":2"
    ".0,\"description\":\"Maximum range in m\",\"type\":\"number\",\"minimum\":-2.0},"
    "\"minimum\":{\"maximum\":2.0,\"description\":\"Minimum range in "
    "m\",\"type\":\"number\",\"minimum\":-2.0}},\"description\":\"Range axis of "
    "correction table\",\"type\":\"object\",\"required\":[\"minimum\",\"maximum\",\"ent"
    "ries\"]}},\"description\":\"Correction parameters for window glass "
    "echoes\",\"required\":[\"amplitude\",\"range\",\"slices\"]}"
    ,
    /* example: */
    "{\"amplitude\":{\"entries\":128,\"maximum\":20,\"minimum\":2},\"slices\":[{\"ampl"
    "itude\":1.5,\"table\":[[[6.23,0.3535,1]],[[5.54,0.72375,1]]]},{\"amplitude"
    "\":2.0,\"table\":[[[6.23,0.3535,1]],[[5.54,0.72375,1]]]}],\"range\":{\"entri"
    "es\":128,\"maximum\":1.5060822940732335,\"minimum\":-1.5060822940732335}}"
};

// Table of all meta data item details records
const MetaDataItemInfo* const RDB_META_DATA_ITEMS[] = {
    &RDB_RIEGL_ANGULAR_NOTCH_FILTER,
    &RDB_RIEGL_ATMOSPHERE,
    &RDB_RIEGL_BEAM_GEOMETRY,
    &RDB_RIEGL_CONTROL_OBJECT_CATALOG,
    &RDB_RIEGL_CONTROL_OBJECT_REFERENCE_FILE,
    &RDB_RIEGL_DETECTION_PROBABILITY,
    &RDB_RIEGL_DEVICE,
    &RDB_RIEGL_DEVICE_GEOMETRY,
    &RDB_RIEGL_DEVICE_GEOMETRY_PASSIVE_CHANNEL,
    &RDB_RIEGL_DEVICE_OUTPUT_LIMITS,
    &RDB_RIEGL_DEVICES,
    &RDB_RIEGL_ECHO_INFO,
    &RDB_RIEGL_EXPONENTIAL_DECOMPOSITION,
    &RDB_RIEGL_GAUSSIAN_DECOMPOSITION,
    &RDB_RIEGL_GEO_TAG,
    &RDB_RIEGL_GEOMETRIC_SCALE_FACTOR,
    &RDB_RIEGL_GEOREFERENCING_PARAMETERS,
    &RDB_RIEGL_ITEM_NAMES,
    &RDB_RIEGL_LICENSES,
    &RDB_RIEGL_MTA_SETTINGS,
    &RDB_RIEGL_NEAR_RANGE_CORRECTION,
    &RDB_RIEGL_NOISE_ESTIMATES,
    &RDB_RIEGL_NOTCH_FILTER,
    &RDB_RIEGL_PIXEL_INFO,
    &RDB_RIEGL_PLANE_PATCH_MATCHING,
    &RDB_RIEGL_PLANE_PATCH_STATISTICS,
    &RDB_RIEGL_PLANE_SLOPE_CLASS_INFO,
    &RDB_RIEGL_POINT_ATTRIBUTE_GROUPS,
    &RDB_RIEGL_POINTCLOUD_INFO,
    &RDB_RIEGL_POSE_ESTIMATION,
    &RDB_RIEGL_POSE_SENSORS,
    &RDB_RIEGL_PULSE_POSITION_MODULATION,
    &RDB_RIEGL_RANGE_STATISTICS,
    &RDB_RIEGL_RECEIVER_INTERNALS,
    &RDB_RIEGL_REFLECTANCE_CALCULATION,
    &RDB_RIEGL_REFLECTANCE_CORRECTION,
    &RDB_RIEGL_SCAN_PATTERN,
    &RDB_RIEGL_SHOT_INFO,
    &RDB_RIEGL_STORED_FILTERS,
    &RDB_RIEGL_SYSTEM_DESCRIPTION,
    &RDB_RIEGL_TEMPERATURE_CALCULATION,
    &RDB_RIEGL_TIME_BASE,
    &RDB_RIEGL_TRAJECTORY_INFO,
    &RDB_RIEGL_TTIP_CONFIGURATION,
    &RDB_RIEGL_VERTEX_INFO,
    &RDB_RIEGL_VOXEL_INFO,
    &RDB_RIEGL_WAVEFORM_AVERAGING_SETTINGS,
    &RDB_RIEGL_WAVEFORM_INFO,
    &RDB_RIEGL_WAVEFORM_SETTINGS,
    &RDB_RIEGL_WINDOW_ANALYSIS,
    &RDB_RIEGL_WINDOW_ECHO_CORRECTION
};
const std::size_t RDB_META_DATA_ITEM_COUNT(
    sizeof(RDB_META_DATA_ITEMS) / sizeof(RDB_META_DATA_ITEMS[0])
);
const std::vector<const MetaDataItemInfo*> RDB_META_DATA_ITEMS_VECTOR(
    RDB_META_DATA_ITEMS, RDB_META_DATA_ITEMS + RDB_META_DATA_ITEM_COUNT
);

}}} // namespace riegl::rdb::pointcloud

#endif // RDB_354CFFB3_BDAA_4E38_98C8_9C09D0C0AD87
